#!/usr/bin/python3
from PyQt5.QtCore import  Qt, QEvent,QSettings
from PyQt5.QtWidgets import QWidget, QApplication,  QPlainTextEdit, QVBoxLayout, QMainWindow, QAction
from PyQt5.QtGui import QIcon, QTextCursor
import sys
from ..cores.stdedit import stdinit
from pygdbmi import gdbmiparser

class Std_termainal(QMainWindow):
    #termin = pyqtSignal(str)  # 0 settext 1 appead 3process


    def __init__(self):
        super(Std_termainal, self).__init__()
        try:
            self.commandslist = []
            self.tracker = 0
            self.watch_id = []
            # os.chdir(os.path.expanduser("~"))
            #        print(os.getcwd())
            # self.name = (str(getpass.getuser()) + "@" + str(socket.gethostname())+ ":" + str(os.getcwd()) + "$ ")
            self.name = ">"
            self.setWindowTitle('sterminal')
            self.setWindowIcon(QIcon.fromTheme("terminal-emulator"))
            self.createStatusBar()
            self.commandfield = QPlainTextEdit()
            self.commandfield.setAttribute(Qt.WA_InputMethodEnabled, False)

            self.commandfield.setStyleSheet(
                "font-family: Noto Sans Mono; font-size: 18pt; background-color: #212821; color:#B0E2FF; padding: 2; border: none;")

            self.commandfield.setLineWrapMode(QPlainTextEdit.NoWrap)
            self.commandfield.setFixedHeight(44)
            self.commandfield.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
            self.commandfield.setAcceptDrops(False)
            self.cursor = self.commandfield.textCursor()

            self.textWindow = QPlainTextEdit(self)  #
            self.textWindow.setStyleSheet(
                "font-family: Noto Sans Mono; font-size: 12pt; background-color: #212121; color: #00FF00; padding: 2; border: none;")
            self.setStyleSheet(mystylesheet(self))
            self.textWindow.setReadOnly(True)
            layout = QVBoxLayout()
            layout.addWidget(self.textWindow)
            layout.addWidget(self.commandfield)
            self.wid = QWidget()
            self.wid.setLayout(layout)
            self.setCentralWidget(self.wid)
            self.setGeometry(0, 0, 600, 500)
            self.commandfield.setPlainText(self.name)

            self.commandfield.setFocus()
            self.copySelectedTextAction = QAction(QIcon.fromTheme("edit-copy"), "Copy", shortcut="Shift+Ctrl+c",
                                                  triggered=self.copyText)
            self.textWindow.addAction(self.copySelectedTextAction)
            self.pasteTextAction = QAction(QIcon.fromTheme("edit-paste"), "Copy", shortcut="Shift+Ctrl+v",
                                           triggered=self.pasteText)
            self.commandfield.addAction(self.pasteTextAction)
            #
            # self.cancelAction = QAction("Cancel", shortcut="Ctrl+c", triggered=self.killProcess)
            # self.textWindow.addAction(self.cancelAction)

            self.commandfield.installEventFilter(self)
            self.commandfield.setCursorWidth(4)
            #        self.textWindow.installEventFilter(self)
            QApplication.setCursorFlashTime(1000)
            self.cursorEnd()
            # self.settings = QSettings("QTerminal", "QTerminal")
            # self.readSettings()
        except:
            stdinit.std_signal_gobal.stdprintln()





    def closeEvent(self, e):
        pass
        #self.writeSettings()

    def cursorEnd(self):
        try:
            # self.name = (str(getpass.getuser()) + "@" + str(socket.gethostname())+ ":" + str(os.getcwd()) + "$ ")
            self.name = ">"
            self.commandfield.setPlainText(self.name)
            cursor = self.commandfield.textCursor()
            cursor.movePosition(11, 0)

            self.commandfield.setTextCursor(cursor)
            self.commandfield.setFocus()
        except:
            stdinit.std_signal_gobal.stdprintln()




    def eventFilter(self, source, event):
        try:
            if source == self.commandfield:
                if (event.type() == QEvent.DragEnter):
                    event.accept()
                    return True
                # elif (event.type() == QEvent.Drop):
                #     print('Drop')
                #     self.setDropEvent(event)
                #     return True
                elif (event.type() == QEvent.KeyPress):
                    cursor = self.commandfield.textCursor()
                    #                print('key press:', (event.key(), event.text()))
                    if event.key() == Qt.Key_Backspace:
                        if cursor.positionInBlock() <= len(self.name):
                            return True
                        else:
                            return False

                    elif event.key() == Qt.Key_Return:
                        self.run()
                        return True

                    elif event.key() == Qt.Key_Left:
                        if cursor.positionInBlock() <= len(self.name):
                            return True
                        else:
                            return False

                    elif event.key() == Qt.Key_Delete:
                        if cursor.positionInBlock() <= len(self.name) - 1:
                            return True
                        else:
                            return False

                    # elif event.modifiers() == Qt.ControlModifier and event.key() == Qt.Key_C:
                    #     self.killProcess()
                    #     return True

                    elif event.key() == Qt.Key_Up:
                        try:
                            if self.tracker != 0:
                                cursor.select(QTextCursor.BlockUnderCursor)
                                cursor.removeSelectedText()
                                self.commandfield.appendPlainText(self.name)

                            self.commandfield.insertPlainText(self.commandslist[self.tracker])
                            self.tracker -= 1

                        except IndexError:
                            self.tracker = 0
                        return True

                    elif event.key() == Qt.Key_Down:
                        try:
                            if self.tracker != 0:
                                cursor.select(QTextCursor.BlockUnderCursor)
                                cursor.removeSelectedText()
                                self.commandfield.appendPlainText(self.name)

                            self.commandfield.insertPlainText(self.commandslist[self.tracker])
                            self.tracker += 1

                        except IndexError:
                            self.tracker = 0
                        return True

                    else:
                        return False
                else:
                    return False
            else:
                return False
        except:
            stdinit.std_signal_gobal.stdprintln()


    def copyText(self):
        try:
            self.textWindow.copy()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def pasteText(self):
        try:
            self.commandfield.paste()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def createStatusBar(self):
        try:
            # sysinfo = QSysInfo()
            # myMachine = "current CPU Architecture: " + sysinfo.currentCpuArchitecture() + " *** " + sysinfo.prettyProductName() + " *** " + sysinfo.kernelType() + " " + sysinfo.kernelVersion()
            myMachine = "GDB"
            self.statusBar().showMessage(myMachine, 0)
        except:
            stdinit.std_signal_gobal.stdprintln()



    def back_text(self,text):
        try:
            response = gdbmiparser.parse_response(text)
            if response["type"] == "notify":
                if response["message"] == "stopped":
                    try:
                        if stdinit.debug_mark_start == 0:
                            stdinit.std_signal_gobal.std_process(0, "DEBUG初始化完成")
                            stdinit.debug_mark_start = 1
                            currant_pro = stdinit.projects_dir + stdinit.project_name
                            if len(stdinit.manual_mark) > 0:
                                for i in stdinit.manual_mark:
                                    if currant_pro in i:
                                        # self.termin.emit("break "+i)
                                        stdinit.std_signal_gobal.std_gdb_cmd(i)

                        else:
                            if response["payload"]["reason"] == "signal-received":
                                stdinit.std_signal_gobal.std_debug_goto(response["payload"]["frame"]["fullname"],
                                                                        response["payload"]["frame"]["func"],
                                                                        "1",
                                                                        response["payload"]["frame"]["line"],
                                                                        response["message"])

                            else:
                                stdinit.std_signal_gobal.std_debug_goto(response["payload"]["frame"]["fullname"],
                                                                        response["payload"]["frame"]["func"],
                                                                        "1",
                                                                        response["payload"]["frame"]["line"],
                                                                        response["message"])


                    except:
                        stdinit.std_signal_gobal.stdprintln()

            else:
                try:
                    self.textWindow.setFocus()
                    out = response["payload"]
                    if type(out) == str:


                        self.textWindow.appendPlainText(out.replace("\\n", "").replace("\\t", " "))
                        if "Error" in out:
                            stdinit.std_signal_gobal.std_gdb_cmd("machinequit")
                            stdinit.std_signal_gobal.std_process(0, "DEBUG初始化失败")
                            self.textWindow.appendPlainText("请检查当前项目platformio.ini文件中相关设置是否正确")
                except:
                    stdinit.std_signal_gobal.stdprintln()

                # self.textWindow.appendPlainText(response["payload"].replace("\\n",""))
            self.cursorEnd()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def run(self):
        #print("started")

        try:

            cmd = self.commandfield.toPlainText()

            self.textWindow.setFocus()
            self.textWindow.appendPlainText(cmd)
            cmd = cmd.replace(self.name, '')
            # self.termin.emit(cmd)
            stdinit.std_signal_gobal.std_gdb_cmd(cmd)
            self.cursorEnd()
            if stdinit.stdenv not in cmd:
                self.commandslist.append(cmd)
        except:
            stdinit.std_signal_gobal.stdprintln()
    def readSettings(self):
        pass
        # if self.settings.contains("commands"):
        #     self.commandslist = self.settings.value("commands")
    def writeSettings(self):
        pass
        #self.settings.setValue("commands", self.commandslist)




def mystylesheet(self):
    return """
QMainWindow{
background-color: #212121; }

QMainWindow:title
{

color: #3465a4;
}

QPlainTextEdit:focus { 
border: none; }

QScrollBar {            
border: 1px solid #2e3436;
background: #292929;
width:8px;
height: 8px;
margin: 0px 0px 0px 0px;
}
QScrollBar::handle {
background: #2e3436;
min-height: 0px;
min-width: 0px;
}
QScrollBar::add-line {
background: #2e3436;
height: 0px;
width: 0px;
subcontrol-position: bottom;
subcontrol-origin: margin;
}
QScrollBar::sub-line {
background: #2e3436;
height: 0px;
width: 0px;
subcontrol-position: top;
subcontrol-origin: margin;
}

QStatusBar {
font-family: Noto Sans Mono; 
font-size: 7pt; 
color: #729fcf;}

"""


def main():
    app = QApplication(sys.argv)
    ex = Std_termainal()
    ex.show()
    #ex.raise_()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtCore import pyqtSignal,QObject
from .stdedit import stdinit
class StdEcho(QObject):
    std_echo_msg = pyqtSignal(int,str)  # 0 settext 1 appead 3process
    # def __init__(self):
    #     pass
        #print(self.abs_path)
        #self.pip_fast()

    def std_message(self,method):#all the time
        stdinit.std_signal_gobal.stdprintln()
    def std_echo(self,staus,msg):#all the time #staus 0 清空
        try:
            self.std_echo_msg.emit(staus, msg)  # 具体实现模仿sterminal的实现方式 绑定
        except:
            stdinit.std_signal_gobal.stdprintln()



# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""

# from six.moves import urllib  # 下载文件
import json
#import zipfile  # 解压文件
# import shutil  # 用于压缩
# import subprocess
import os,stat
import sys
import re
# import socket
#
# import zipfile

from PyQt5.QtWidgets import *
from PyQt5.QtCore import QStringListModel,pyqtSignal,QThread
# from function.conf import res
from .stdmsg import reso

from PyQt5.QtWidgets import QMessageBox

import threading
from .stdedit import stdinit
import time

# fo = open("./appearance/stduino_json/library.json", mode='r', encoding='UTF-8')
# st = fo.read()
# fo.close()
# libraries = json.loads(st)

class stdthread(QThread):
    std_signal = pyqtSignal(str)
    def __init__(self):
        super(stdthread, self).__init__()
    def run(self):
        try:
            # a = (1, 2)
            # self.mysignal.emit(a)  # 发射自定义信号
            if stdinit.callch == 1:
                self.install_pio()
            elif stdinit.callch == 2:
                self.install_p()
        except:
            stdinit.std_signal_gobal.stdprintln()

    def install_pio(self):
        try:
            if stdinit.Pio_install.is_installed_pio():
                stdinit.std_signal_gobal.std_process(1, reso.package_uninstall)

                if stdinit.Pio_install.pio_uninstall():
                    self.std_signal.emit(reso.install_lib)
                    # self.install.setText(reso.install_lib)
                    stdinit.std_signal_gobal.std_process(0, reso.package_uninstall)
                    stdinit.std_signal_gobal.std_echo_msg(0, "卸载" + stdinit.package_choose_name + "成功！")
                else:
                    self.std_signal.emit(reso.install_lib)
                    stdinit.std_signal_gobal.std_process(0, reso.package_uninstall)
                    stdinit.std_signal_gobal.std_echo_msg(0, "卸载" + stdinit.package_choose_name + "失败！")


            else:
                stdinit.std_signal_gobal.std_process(1, reso.downloading)
                if stdinit.Pio_install.pio_install():
                    # self.install.setText(reso.package_uninstall)
                    self.std_signal.emit(reso.package_uninstall)
                    stdinit.std_signal_gobal.std_process(0, reso.downloading)
                    stdinit.std_signal_gobal.std_echo_msg(0, "安装" + stdinit.package_choose_name + "成功！")
                else:
                    self.std_signal.emit(reso.install_lib)
                    stdinit.std_signal_gobal.std_process(0, reso.downloading)
                    stdinit.std_signal_gobal.std_echo_msg(0, "安装" + stdinit.package_choose_name + "失败！")
        except:
            stdinit.std_signal_gobal.stdprintln()

    def install_p(self):
        try:
            for i in range(0, 5):

                time.sleep(1)

        except:
            stdinit.std_signal_gobal.stdprintln()


class StdAddPackages(QWidget):  # 安装库文件

    add_lib_s = pyqtSignal(int,int,str)
    def __init__(self):
        try:
            super(StdAddPackages, self).__init__()
            self.add_lib_s_load_f = 0
            self.find_more_p = 1
            self.pio_libs = []
            self.pio_libs.append(0)
            self.lib_list = []
            self.initUi()
        except:
            stdinit.std_signal_gobal.stdprintln()

    def initUi(self):
        try:
            self.bigEditor = QTextEdit()
            self.bigEditor.setReadOnly(True)


            #self.createGridGroupBox()
            self.createGridGroupBox2()
            self.creatVboxGroupBox()
            self.creatVboxGroupBox2()
            mainLayout = QVBoxLayout()
            hboxLayout = QHBoxLayout()
            hboxLayout.addWidget(self.vboxGroupBox2)
            hboxLayout.addWidget(self.vboxGroupBox)

            #the_window.add_lib_s_load.connect(self.libr_load)



            self.find_more = QPushButton(reso.lib_find_more, self)
            self.find_more.clicked.connect(self.find_mores)
            self.find_more.setEnabled(False)
            self.install = QPushButton(reso.install_lib, self)
            self.install.setDisabled(True)
            self.install.clicked.connect(self.install_url)
            self.install_return = QPushButton(reso.library_installer, self)
            self.install_return.clicked.connect(self.install_ret)
            #self.install.setFixedWidth(120)
            #self.pro1 = QProgressBar(self)
            # self.pro1.setOrientation(Qt.Horizontal)
            # self.pro1.setMinimum(0)
            # self.pro1.setMaximum(0)
            #self.pro1.setVisible(False)

            glayout = QGridLayout()
            glayout.addWidget(self.install, 0, 0)
            glayout.addWidget(self.find_more, 0, 1)
            glayout.addWidget(self.install_return, 0, 2)


            # 准备四个控件
            gwg = QWidget()
            # 使用四个控件设置局部布局
            gwg.setLayout(glayout)
            # 将四个控件添加到全局布局中


            #mainLayout.addWidget(self.gridGroupBox)
            mainLayout.addLayout(hboxLayout)
            mainLayout.addWidget(self.gridGroupBox2)
            #mainLayout.addWidget(self.pro1)
            mainLayout.addWidget(gwg)
            #mainLayout.addWidget(self.install_return)
            self.setLayout(mainLayout)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def libr_load(self):
        try:
            if self.add_lib_s_load_f == 0:
                self.add_lib_s_load_f = 1
                t1 = threading.Thread(target=self.library_load, name='li_load1')
                t1.setDaemon(True)
                t1.start()

        except:
            stdinit.std_signal_gobal.stdprintln()




    def createGridGroupBox2(self):
        try:
            self.gridGroupBox2 = QGroupBox(reso.lib_version)
            self.layout = QGridLayout()
            # nameLabel = QLabel("发布版本时间")
            self.vsion = QCheckBox(reso.select_version, self)
            self.vsion.setChecked(True)
            self.vsion.setEnabled(False)
            # self.vsion.stateChanged.connect(self.check_vsion)
            self.combobox_1 = QComboBox(self)
            self.combobox_1.setDisabled(True)
            self.layout.setSpacing(10)
            self.layout.addWidget(self.vsion, 1, 0)
            self.combobox_1.addItem("")
            self.layout.addWidget(self.combobox_1, 1, 1)
            self.layout.setColumnStretch(1, 10)
            self.gridGroupBox2.setLayout(self.layout)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def creatVboxGroupBox(self):
        try:
            self.vboxGroupBox = QGroupBox(reso.lib_detail)
            layout = QVBoxLayout()
            self.bigEditor.setPlainText(reso.lib_select)
            layout.addWidget(self.bigEditor)
            self.vboxGroupBox.setLayout(layout)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def creatVboxGroupBox2(self):

        try:
            self.vboxGroupBox2 = QGroupBox(reso.package_list)
            layout = QVBoxLayout()
            self.model_1 = QStringListModel(self)
            self.listview_1 = QListView(self)  # python3
            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)

            self.listview_1.setEditTriggers(QAbstractItemView.NoEditTriggers)
            #self.listview_1.setFixedWidth(600)
            #self.listview_1.setFixedHeight(400)
            self.listview_1.clicked.connect(self.clicked)
            layout.addWidget(self.listview_1)
            self.vboxGroupBox2.setLayout(layout)
                    # self.listview_1.doubleClicked.connect(lambda: self.change_func(self.listview_1))
                    # self.listview_1.doubleClicked.connect(self.cs)

                    #self.listview_1.clicked.connect(self.clicked)
        except:
            self.lib_list = [reso.no_net]
            self.model_1 = QStringListModel(self)
            self.listview_1 = QListView(self)  # python3


            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)
            self.listview_1.setEditTriggers(QAbstractItemView.NoEditTriggers)
            #self.listview_1.setFixedWidth(600)
            #self.listview_1.setFixedHeight(400)

            # self.item_list = ['item %s' % i for i in range(51)]  # 1
            stdinit.std_signal_gobal.stdprintln()

    def library_load(self):
        try:
            self.install_return.setText("Finding File...")
            lib_show = "PlatformIO\nby platformio    |    License:Apache 2.0"
            self.lib_list.append(lib_show + "\n")
            self.model_1.setStringList(self.lib_list)

            self.listview_1.setModel(self.model_1)
            self.install_return.setText(reso.library_installer)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def thread_return(self,strin):
        try:
            self.install.setText(strin)
            self.install.setEnabled(True)
            self.listview_1.setEnabled(True)
        except:
            stdinit.std_signal_gobal.stdprintln()



    def install_ret(self):#退出安装界面
        try:
            stdinit.std_signal_gobal.std_main_ui_change(3)
            # the_window.m_stackedLayout.setCurrentIndex(0)
            # the_window.liAct2.setEnabled(True)
        except:
            stdinit.std_signal_gobal.stdprintln()
    def clicked(self):#(self, qModelIndex):
        #self.my_thread.mysignal.connect(self.zhi)  # 自定义信号连接
        try:
            #n_lin=qModelIndex.row()
            descrabe = "A professional collaborative platform for embedded development. Cross-platform IDE and Unified Debugger. Static Code Analyzer and Remote Unit Testing. Multi-platform and Multi-architecture Build System. Firmware File Explorer and Memory Inspection. IoT, Arduino, CMSIS, ESP-IDF, FreeRTOS, libOpenCM3, mbedOS, Pulp OS, SPL, STM32Cube, Zephyr RTOS, ARM, AVR, Espressif (ESP8266/ESP32), FPGA, MCS-51 (8051), MSP430, Nordic (nRF51/nRF52), NXP i.MX RT, PIC32, RISC-V, STMicroelectronics (STM8/STM32), Teensy."#self.pio_libs[n_lib]["items"][n_lin]['description']+ "\n\nTag:"
            descrabe=descrabe+"\n\nHomepage:https://platformio.org"
            self.combobox_1.clear()
            stdinit.package_choose_name= "platformio"
            self.combobox_1.addItem("6.1.4")
            if stdinit.Pio_install.is_installed_pio():
                self.install.setDisabled(False)
                self.install.setText(reso.package_uninstall)
                pass
            else:
                self.install.setDisabled(False)
                self.install.setText(reso.install_lib)

            self.bigEditor.setText(descrabe)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def remove_readonly(self,func, path, _):
        try:
            # "Clear the readonly bit and reattempt the removal"
            os.chmod(path, stat.S_IWRITE)
            func(path)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def install_url(self):

        try:
            self.install.setEnabled(False)
            self.listview_1.setEnabled(False)
            if stdinit.Pio_install.is_installed_pio():
                ok = QMessageBox.information(self,
                                             "提示",
                                             "确认要卸载" + stdinit.package_choose_name + "？",
                                             QMessageBox.Yes, QMessageBox.No)
                if ok == QMessageBox.No:
                    return 0
            self.my_thread = stdthread()  # 步骤2. 主线程连接子线,同时传递一个值给子线程
            self.my_thread.std_signal.connect(self.thread_return)
            stdinit.callch = 1
            self.my_thread.start()  # 步骤3 子线程开始执行run函数
        except:
            stdinit.std_signal_gobal.stdprintln()
        #t1 = threading.Thread(target=pay_ticket, name='th1')



    def schedule(self,blocknum, blocksize, totalsize):
        try:
            # """
            #         blocknum:当前已经下载的块
            #         blocksize:每次传输的块大小
            #         totalsize:网页文件总大小
            #         """
            per = 100 * blocknum * blocksize / totalsize

            if per > 100:
                per = 100
                self.install_return.setText("Installing Package...")

            self.add_lib_s.emit(2, int(per), "")  # 2进度条数值
        except:
            stdinit.std_signal_gobal.stdprintln()

    def find_mores(self):
        try:
            self.install_return.setText("Finding File...")
            self.find_more_p = self.find_more_p + 1
            self.pio_libs.append(self.lib.lib_search(self.find_more_p))
            ii = len(self.pio_libs[self.find_more_p]['items'])
            i = 0
            while i < ii:
                self.lib_list.append(self.pio_libs[self.find_more_p]['items'][i]['name'] + "\ndescription:" +
                                     self.pio_libs[self.find_more_p]['items'][i]['description'] + "\n")  #
                i = i + 1
            self.model_1.setStringList(self.lib_list)

            self.listview_1.setModel(self.model_1)
            self.install_return.setText(reso.library_installer)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def check_vsion(self):
        try:
            if self.vsion.isChecked():
                self.combobox_1.setDisabled(False)
            else:
                self.combobox_1.setDisabled(True)
        except:
            stdinit.std_signal_gobal.stdprintln()





# class Model(object):
#     def logic(self):
#         data = "Got it"
#         print("Model: Crunching data as per business logic")
#         return data
#
#
# class View(object):
#     def updata(self, data):
#         print("View:Updating the view with results: ", data)
#
#
# class Controller(object):
#     def __init__(self):
#         self.model = Model()
#         self.view = View()
#
#     def interface(self):
#         print("Controller: Relayed the Client asks")
#         data = self.model.logic()
#         self.view.updata(data)

if __name__ == '__main__':  # main函数
    app = QApplication(sys.argv)

    try:


        d = StdAddPackages()
        d.show()

        # the_window.fileName = reso.lastfile
        # the_window.loadFile(reso.lastfile)



    except:
        pass
    sys.exit(app.exec_())





from PyQt5.QtCore import pyqtSignal,QObject
import sys,traceback
import requests
from function.cores.stdmsg import reso
class StdSinal(QObject):
    try:
        project_path_observer=pyqtSignal(str)
        open_project_path_observer=pyqtSignal(str)
        find_change = pyqtSignal(int, str, str, bool)
        text_change = pyqtSignal()
        back_color = pyqtSignal(str, int)
        main_ui_change = pyqtSignal(int)
        process_p = pyqtSignal(int, str)  # 0 settext 1 appead 3process
        echo_msg = pyqtSignal(int, str)  # 0 settext 1 appead 3process
        work_space = pyqtSignal()  #
        find_staus_change = pyqtSignal()  #
        jump_to_look = pyqtSignal(str, int, str)
        debug_goto = pyqtSignal(str, str, str, str, str)  # fullname func number line
        gdb_cmd = pyqtSignal(str)  # fullname func number line
        jump_init = pyqtSignal()
        upload_list_change = pyqtSignal()
        file_modify = pyqtSignal()
        make_signal = pyqtSignal(int)  # 0 settext 1 appead 3process
        upload_Close_check = pyqtSignal()  # 0 settext 1 appead 3process
        upload_rts_dtr= pyqtSignal()  # 0 settext 1 appead 3process
        stdenvir_init=pyqtSignal()
        comment_uncomment=pyqtSignal()
        main_ui_setview = pyqtSignal(int)
    except:
        print(traceback.format_exc())
    def std_main_ui_setview(self,i):
        self.main_ui_setview.emit(i)
    def all_stdenvir_init(self):
        self.stdenvir_init.emit()

    def std_project_path_observer(self,path):
        self.project_path_observer.emit(path)

    def std_comment_uncomment(self):  # all the time
        try:
            self.comment_uncomment.emit()  # 具体实现模仿sterminal的实现方式 绑定
        except:
            self.stdprintln()

    def is_connected(self):
        try:
            requests.get("https://www.baidu.com", timeout=2)
        except:
            self.std_echo_msg(1, reso.no_net)
            return False
        return True


    def stdprintln(self):
        try:
            print(traceback.format_exc())
            #print("Unexpected error:", sys.exc_info()[1])  # 错误内容
            self.echo_msg.emit(1, "Unexpected error:" + str(sys.exc_info()[1]))


            #self.echo_msg.emit(1, traceback.format_exc())
            self.echo_msg.emit(1,"当你看到这段报错时，请根据使用情况进行判断是否影响正常使用，若严重影响使用请及时反馈，我将于2021年6月份后根据具体情况逐一修复；可通过软件左下角反馈功能反馈亦可至www.stduino.com进行反馈！\n 感谢您的使用，让我们一起在磨砺中成长。")
        except:
            print(traceback.format_exc())


        #还可以将信息写入到文件 发布后用这个方式进行收集错误
        #traceback.print_exc(file=open(‘error.txt','a+'))
        #print("Unexpected error:", sys.exc_info()[1])  # 错误内容
        # QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdfile00040！\n You can search it in stduino.com",QMessageBox.Yes)
    def std_upload_rts_dtr(self):  # all the time
        try:
            self.upload_rts_dtr.emit()  # 具体实现模仿sterminal的实现方式 绑定
        except:
            self.stdprintln()
    def std_process(self, staus, text):  # all the time
        try:
            self.process_p.emit(staus, text)  # 具体实现模仿sterminal的实现方式 绑定
        except:
            self.stdprintln()

    def std_find(self,idd, changetext, findtext, afind):
        try:
            self.find_change.emit(idd, changetext, findtext, afind)
        except:
            self.stdprintln()

    def std_text_change(self):
        try:
            self.text_change.emit()
        except:
            self.stdprintln()

    def std_back_color(self,str, id):  # startsetcolor
        try:
            self.back_color.emit(str, id)
        except:
            self.stdprintln()

    def std_main_ui_change(self,id):
        try:
            self.main_ui_change.emit(id)
        except:
            self.stdprintln()

    def std_echo_msg(self,staus,msg):#all the time #staus 0 清空
        try:
            self.echo_msg.emit(staus,msg) #具体实现模仿sterminal的实现方式 绑定
        except:
            self.stdprintln()


    def std_work_space(self):
        try:
            self.work_space.emit()
        except:
            self.stdprintln()

    def std_find_staus_change(self):
        try:
            self.find_staus_change.emit()
        except:
            self.stdprintln()

    def std_jump_to_look(self,path,line,text):
        try:
            self.jump_to_look.emit(path,line,text)
        except:
            self.stdprintln()

    def std_debug_goto(self,fullname,func,number,line,message):
        try:
            self.debug_goto.emit(fullname,func,number,line,message)
        except:
            self.stdprintln()

    def std_gdb_cmd(self,cmd):
        try:
            self.gdb_cmd.emit(cmd)
        except:
            self.stdprintln()

    def std_jump_init(self):
        try:
            self.jump_init.emit()
        except:
            self.stdprintln()

    def std_upload_list_change(self):
        try:
            self.upload_list_change.emit()
        except:
            self.stdprintln()

    def std_file_modify(self):
        try:
            self.file_modify.emit()
        except:
            self.stdprintln()
    def std_make_signal(self,id):
        try:
            self.make_signal.emit(id)
        except:
            self.stdprintln()

    def std_upload_Close_check(self):
        try:
            self.upload_Close_check.emit()
        except:
            self.stdprintln()






# -*- coding: utf-8 -*-

"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
# from pygdbmi import gdbmiparser
import subprocess
import json
import os
from ..cores.stdedit import stdinit
#from function.cores.stdedit import stdinit

#调用前先判断是否已经正常安装pio
class PioRunManage():
    def __init__(self):
        if stdinit.platform_is == "Win":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio" + '"'

        elif stdinit.platform_is == "Linux":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'

        elif stdinit.platform_is == "Darwin":
            self.pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'




    def pio_run(self):#待完善make#分离目录获取第 7个值  C:\Users\debug\Documents\Stduino\Projects
        try:
            stdinit.std_signal_gobal.std_echo_msg(0, "")
            if stdinit.project_name==None:
                stdinit.std_signal_gobal.std_echo_msg(1,"软件正在初始化,请稍等片刻~")
                return 0
            else:
                #pro_curt = '"' + stdinit.projects_dir + stdinit.project_name + '"'
                pro_curt = '"' + stdinit.projects_dir + stdinit.project_name + '"'
                stdinit.run_platforms=0
                # print(os.path.join(pro_curt,"stduino.ini"))

                if os.path.isfile(stdinit.projects_dir + stdinit.project_name +"/stduino.ini"):
                    cmd = "C:/Users/debug/PycharmProjects/pyst123test/venv/Scripts/python.exe D:/pythonpro/stduinocore/src/stduino/function/cores/commands/commands.py run build -d " + pro_curt
                # if stdinit.run_platforms==1:
                #
                #     os.chdir("C:/Users/debug/.stduino/packages/framework-arduino-w806/tools/stduino/")  # 通过更改当前运行目录
                #     cmd="C:/Users/debug/.stduino/packages/python3/python C:/Users/debug/.stduino/packages/framework-arduino-w806/tools/stduino/test.py"

                else:

                    if stdinit.std_make_board == None:
                        cmd = self.pio_env + " run -d " + pro_curt
                    else:
                        cmd = self.pio_env + " run -e " + stdinit.std_make_board + " -d " + pro_curt

                    # pio run -e uno -t upload.

            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            for line in iter(proc.stdout.readline, b''):
                # s1 = str(line)#
                try:
                    s1 = str(line, encoding='gbk')
                except:
                    s1 = str(line)
                # print(s1)
                # response = gdbmiparser.parse_response(s1)
                # print(response)
                stdinit.std_signal_gobal.std_echo_msg(1, s1)
                # print(s1)
                if not subprocess.Popen.poll(proc) is None:
                    if line == "":
                        break
            proc.stdout.close()





        except:
            stdinit.std_signal_gobal.stdprintln()
    def pio_clean(self):#待完善
        try:
            stdinit.std_signal_gobal.std_echo_msg(0, "")
            if stdinit.project_name==None:
                stdinit.std_signal_gobal.std_echo_msg(1,"软件正在初始化,请稍等片刻~")
                return 0
            else:
                pro_curt='"'+stdinit.projects_dir + stdinit.project_name+'"'
                if stdinit.std_make_board == None:
                    cmd = self.pio_env+" run -d " + pro_curt + " -t clean"
                else:
                    cmd = self.pio_env+" run -e " + stdinit.std_make_board + " -d " + pro_curt + " -t clean"
                proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
                for line in iter(proc.stdout.readline, b''):
                    try:
                        s1 = str(line, encoding='gbk')
                    except:
                        s1 = str(line)
                    stdinit.std_signal_gobal.std_echo_msg(1, s1)
                    if not subprocess.Popen.poll(proc) is None:
                        if line == "":
                            break
                proc.stdout.close()



        except:
            stdinit.std_signal_gobal.stdprintln()

    def pio_upload(self):#待完善
        try:
            stdinit.std_signal_gobal.std_echo_msg(0, "")
            if stdinit.project_name==None:
                stdinit.std_signal_gobal.std_echo_msg(1,"软件正在初始化,请稍等片刻~")
                return 0
            else:

                if stdinit.current_upload_m=="serial" and stdinit.platform=="ststm32":
                    stdinit.std_signal_gobal.std_upload_rts_dtr()


                pro_curt = '"' + stdinit.projects_dir + stdinit.project_name + '"'
                if stdinit.std_make_board == None:
                    cmd = self.pio_env+" run -d " + pro_curt + " -t upload"

                else:
                    cmd = self.pio_env+" run -e " + stdinit.std_make_board + " -d " + pro_curt + " -t upload"
                proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
                for line in iter(proc.stdout.readline, b''):
                    try:
                        s1 = str(line, encoding='gbk')
                    except:
                        s1 = str(line)
                    stdinit.std_signal_gobal.std_echo_msg(1, s1)
                    if not subprocess.Popen.poll(proc) is None:
                        if line == "":
                            break
                proc.stdout.close()
        except:
            stdinit.std_signal_gobal.stdprintln()


    def pio_device_list(self):#待完善
        try:

            cmd = self.pio_env+" device list --json-output"
            rest = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)  # 使用管道
            try:
                data = str(rest.stdout.read(), encoding='gbk')
            except:
                data = str(rest.stdout.read())

            rest.stdout.close()
            stdinit.device = json.loads(data)
            return True
        except:
            stdinit.std_signal_gobal.stdprintln()



#from pygdbmi.gdbcontroller import GdbController
#gdbmi = GdbController("C:\stwork\\tool\\test32\packages\STM32\\tools\xpack-arm-none-eabi-gcc\9.2.1-1.1\\bin\arm-none-eabi-gdb.exe", [""]) #第一个参数指定gdb，第二个参数指定elf文件
# response = gdbmi.write('set mem inaccessible-by-default off')
# response = gdbmi.write('set arch riscv:rv32')
# response = gdbmi.write('target remote localhost:3333', 5) #5为timeout时间
# response = gdbmi.write('load', 6)

import subprocess
import os
#from pygdbmi.gdbcontroller import GdbController
import _thread
from ..cores.stdedit import stdinit
from PyQt5.QtWidgets import *
from PyQt5.QtCore import pyqtSignal
from PyQt5.QtGui import QIcon
from PyQt5 import QtCore

import sys
from PyQt5.QtCore import QStringListModel
import time
import re
global debug_staus2
debug_staus2=0
global debug_staus
debug_staus=0
global debug_staus_st
debug_staus_st=0


class StDebug(QWidget):
    debug_sig = pyqtSignal(int)  # 0 settext 1 appead 3process
    debug_return = pyqtSignal(str)  # 0 settext 1 appead 3process
    def __init__(self):
        super(StDebug, self).__init__()
        try:
            # self.setWindowOpacity(0.7)
            # self.setGeometry(50, 50, 50, 30)

            # self.setFixedSize(420, 220)
            self.setWindowTitle('StdDebug View')
            self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
            stdinit.std_signal_gobal.gdb_cmd.connect(self.st_gdb_cmd)
            # self.setWindowIcon(QIcon("appearance/img/st.PNG"))

            # 全局布局（2中）：这里选择水平布局
            # wlayout = QHBoxLayout()
            wlayout = QVBoxLayout()

            # 局部布局：水平，垂直，网格，表单
            glayout = QGridLayout()

            self.de_start = QToolButton()  # 无背景
            self.de_start.clicked.connect(self.st_piodebug)
            self.de_start.setIcon(QIcon("./appearance/img/startup.png"))  # setIcon(QIcon("appearance/img/img_dot.png"))
            self.de_start.setToolTip("Start Debugging")

            self.de_continue = QToolButton()  # 无背景
            self.de_continue.clicked.connect(self.de_continue1)
            self.de_continue.setIcon(
                QIcon("./appearance/img/continue.png"))  # setIcon(QIcon("appearance/img/img_dot.png"))
            self.de_continue.setToolTip("Continue")
            self.de_continue.setDisabled(True)
            # de_pause = QToolButton()  # 无背景
            # de_pause.setIcon(QIcon("./appearance/img/pause.png"))  换图片
            self.debug_step_over = QToolButton()  # 无背景
            self.debug_step_over.clicked.connect(self.debug_step_over1)
            self.debug_step_over.setDisabled(True)

            self.debug_step_over.setIcon(QIcon("./appearance/img/debug_step_over.png"))
            self.debug_step_over.setToolTip("Step over")
            self.debug_step_into = QToolButton()  # 无背景
            self.debug_step_into.clicked.connect(self.debug_step_into1)
            self.debug_step_into.setDisabled(True)
            self.debug_step_into.setIcon(QIcon("./appearance/img/debug_step_into.png"))
            self.debug_step_into.setToolTip("Step into")
            self.debug_step_out = QToolButton()  # 无背景
            self.debug_step_out.setDisabled(True)
            self.debug_step_out.clicked.connect(self.debug_step_out1)
            self.debug_step_out.setIcon(QIcon("./appearance/img/debug_step_out.png"))
            self.debug_step_out.setToolTip("Step out")
            self.restart = QToolButton()  # 无背景
            self.restart.setDisabled(True)
            self.restart.clicked.connect(self.debug_restart1)
            self.restart.setIcon(QIcon("./appearance/img/restart.png"))
            self.restart.setToolTip("Restart")
            self.de_stop = QToolButton()  # 无背景
            self.de_stop.setDisabled(True)
            self.de_stop.clicked.connect(self.pio_de_stop2)
            self.de_stop.setIcon(QIcon("./appearance/img/stop.png"))
            self.de_stop.setToolTip("Stop")

            # 实例化列表视图
            listview = QListView()

            # 实例化列表模型，添加数据
            slm = QStringListModel()
            self.qList = ['gdb常用指令', 'b 函数名/行数', 'n/next', 'l/list', 'list 行号', 'list 函数名', 's/step', 'set var 变量名=值',
                          'bt(backtrace)', 'continue/c', 'whatis 变量名', 'info watchpoints', 'info breakpoints',
                          'info all-registers', 'd/delete', 'display var', 'wiki.stduino.com for detail']

            # 设置模型列表视图，加载数据列表
            slm.setStringList(self.qList)

            # 设置列表视图的模型
            listview.setModel(slm)

            glayout.addWidget(self.de_start, 0, 0)
            glayout.addWidget(self.de_continue, 1, 0)
            glayout.addWidget(self.debug_step_over, 1, 1)
            glayout.addWidget(self.debug_step_into, 1, 2)
            glayout.addWidget(self.debug_step_out, 1, 3)
            glayout.addWidget(self.restart, 1, 4)
            glayout.addWidget(self.de_stop, 1, 5)
            # 准备四个控件
            gwg = QWidget()
            # 使用四个控件设置局部布局
            gwg.setLayout(glayout)
            # 将四个控件添加到全局布局中
            wlayout.addWidget(gwg)
            wlayout.addWidget(listview)

            self.setLayout(wlayout)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def startgdbserver(self,threadName, delay):
        try:
            global debug_staus
            global debug_staus_st

            pos = os.path.abspath('.')
            path = pos + "\\tool\Servers\STM32CubeProgrammer\\binst"
            os.chdir(path)  # 通过更改当前运行目录F:\BaiduNetdiskDownload\stpython\stdemo\main
            cmd = "ST-LINK_CLI -c -P"
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            os.chdir(pos)
            i = 0
            for line in iter(proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='utf-8')
                except:
                    pass
                a = re.search('Unable', s1)
                if a == None:
                    pass
                else:
                    QMessageBox.warning(self, "Can't DEBUG", "Please check that your stlink is connected properly?",
                                        QMessageBox.Yes)
                    # self.debug_return.emit("Please check that your stlink is connected properly?")
                    debug_staus = 1
                    return 1
            os.chdir(pos + "/tool/Servers/ST-LINK_gdbserver")
            cmd = "ST-LINK_gdbserver -e -p 61237 -r 15 -d"
            self.proc_server = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
            os.chdir(pos)
            debug_staus = 0
            for line in iter(self.proc_server.stdout.readline, b''):
                pass
                s1 = str(line, encoding='gbk')
                # print(s1)
                if "Waiting for connection" in s1:
                    debug_staus_st = 1
                else:
                    debug_staus_st = 0

                # else:
                #     QMessageBox.warning(self, "Can't DEBUG", "ST-LINK: Could not verify ST device! Abort connection",
                #                         QMessageBox.Yes)
                # print(s1)
            #         pass
            # self.startgdb("st",1)
            return 0
        except:
            stdinit.std_signal_gobal.stdprintln()



    def st_gdb_cmd(self,cmd):

        try:

            if not cmd.endswith("\n"):
                cmd = cmd + "\n"
            self.proc.stdin.write(cmd.encode())
            self.proc.stdin.flush()
            if cmd == "quit\n" or cmd == "q\n":
            #if cmd == "y\n":
                self.debug_sig.emit(1)  #指令发送进行关闭时触发
                self.proc.terminate()
                self.pio_de_stop1()
                time.sleep(1)
            if cmd == "machinequit\n":
                self.proc.terminate()
                self.pio_de_stop1()

                #os.popen('taskkill /IM ST-LINK_gdbserver.exe /F')

        except:
            stdinit.std_signal_gobal.stdprintln()

    def startgdb(self,threadName, delay):
        try:
            # self.thread.trigger.connect(self.ListShow)
            pos = os.path.abspath('.')  # C:/stwork/stdemo2019827/tool/gcc/bin/arm-none-eabi-gdb.exe C:/stwork/stdemo2019827/tool/test/bin/test.elf
            cmd = pos + "/tool/gcc/bin/arm-none-eabi-gdb.exe " + pos + "/tool/test/bin/test.elf --quiet"

            # cmd = "C:/stwork/stdemo2019827/tool/gcc/bin/arm-none-eabi-gdb.exe C:/stwork/stdemo2019827/tool/test/bin/test.elf --quiet"

            self.proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True,
                                         stdin=subprocess.PIPE)
            for line in iter(self.proc.stdout.readline, b''):
                s1 = str(line, encoding='gbk')
                if s1 == "":
                    pass
                elif "answered Y" in s1:
                    self.debug_return.emit("Successful execution!")
                else:
                    self.debug_return.emit(s1)
                    # print(s1)

            # self.gdbmi = GdbController("C:/stwork/stdemo2019827/tool/gcc/bin/arm-none-eabi-gdb.exe",["C:/stwork/stdemo2019827/tool/test/bin/test.elf"])  # 第一个参数指定gdb，第二个参数指定elf文件
            # #
            # print(self.gdbmi.get_subprocess_cmd())  # print actual command run as subprocess
            # response = self.gdbmi.write('target remote localhost:65539')  # 5为timeout时间
            # print(response)
            # response = self.gdbmi.write('load')
            # print(response)
            # response = self.gdbmi.write('b setup')
            # print(response)
            # response = self.gdbmi.write('b loop')
            # print(response)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def de_continue1(self):
        try:
            self.st_gdb_cmd('c\n')
            # response = self.gdbmi.write('c', 6)
            # print(response)
            pass
        except:
            stdinit.std_signal_gobal.stdprintln()

    def debug_step_over1(self):
        try:
            self.st_gdb_cmd('n\n')
            # response = self.gdbmi.write('n', 6)
            # print(response)

        except:
            stdinit.std_signal_gobal.stdprintln()

    def debug_step_into1(self):
        try:
            self.st_gdb_cmd('s\n')
            # response = self.gdbmi.write('s', 6)
            # print(response)
            pass

        except:
            stdinit.std_signal_gobal.stdprintln()

    def debug_step_out1(self):
        try:
            self.st_gdb_cmd('finish\n')
            # response = self.gdbmi.write('finish', 6)
            # print(response)
            pass

        except:
            stdinit.std_signal_gobal.stdprintln()

    def debug_restart1(self):
        try:
            # self.st_gdb_cmd('finish')
            # response = self.gdbmi.write('fin', 6)
            # self.st_debug()
            # print(response)
            self.st_gdb_cmd('monitor reset\n')

        except:
            stdinit.std_signal_gobal.stdprintln()

    def pio_de_stop1(self):
        try:
            # response = self.gdbmi.write('quit', 6)
            # print(response)
            self.debug_step_out.setDisabled(True)
            self.debug_step_into.setDisabled(True)
            self.debug_step_over.setDisabled(True)
            self.de_continue.setDisabled(True)
            self.restart.setDisabled(True)
            self.de_stop.setDisabled(True)
            self.de_start.setDisabled(False)

        except:
            stdinit.std_signal_gobal.stdprintln()


    def pio_de_stop2(self):
        try:
            self.st_gdb_cmd('quit')
            self.de_start.setDisabled(False)
            self.debug_step_out.setDisabled(True)
            self.debug_step_into.setDisabled(True)
            self.debug_step_over.setDisabled(True)
            self.de_continue.setDisabled(True)
            self.restart.setDisabled(True)
            self.de_stop.setDisabled(True)


        except:
            stdinit.std_signal_gobal.stdprintln()

    def return_de_staus2(self,i):
        pass
    def gdb_init(self):
        try:
            cmd = "target remote localhost:61237\n"
            self.st_gdb_cmd(cmd)
            cmd = "load\n"
            self.st_gdb_cmd(cmd)
            cmd = "b setup\n"
            self.st_gdb_cmd(cmd)
            cmd = "b loop\n"
            self.st_gdb_cmd(cmd)
        except:
            stdinit.std_signal_gobal.stdprintln()


    #You can't do that
    def pio_cmd_init(self,cmd):
        try:
            cmd = "b setup\n"
            self.st_gdb_cmd(cmd)
            cmd = "b loop\n"
            self.st_gdb_cmd(cmd)
            cmd = "c\n"
            self.st_gdb_cmd(cmd)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def pio_de_start(self):
        try:
            self.de_start.setDisabled(True)
            self.debug_step_out.setDisabled(False)
            self.debug_step_into.setDisabled(False)
            self.debug_step_over.setDisabled(False)
            self.de_continue.setDisabled(False)
            self.restart.setDisabled(False)
            self.de_stop.setDisabled(False)
        except:
            stdinit.std_signal_gobal.stdprintln()



    def piodebug(self,t,tt):#stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio
        try:
            self.debug_sig.emit(0)  # 指令发送进行关闭时触发
            if stdinit.platform_is == "Win":
                pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio" + '"'

            elif stdinit.platform_is == "Linux":
                pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'


            elif stdinit.platform_is == "Darwin":
                pio_env = '"' + stdinit.stdenv + "/.stduino/packages/pioenv/bin/pio" + '"'
            else:
                return 0


            piogdb_path = pio_env+" debug -d " +'"' +  stdinit.projects_dir+ stdinit.project_name+ '"' + " --interface=gdb --interpreter mi -x .pioinit"#--interpreter mi
            self.proc = subprocess.Popen(piogdb_path, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True,
                                         stdin=subprocess.PIPE)
            for line in iter(self.proc.stdout.readline, b''):
                try:
                    s1 = str(line, encoding='utf-8')
                    self.debug_return.emit(s1)
                except:
                    print("debug:error")
            pass
        except:
            stdinit.std_signal_gobal.std_process(0, "DEBUG初始化失败")
            self.debug_return.emit("DEBUG初始化失败!")
            stdinit.std_signal_gobal.stdprintln()


    def st_piodebug(self):
        try:
            if stdinit.init_path == None:
                stdinit.std_signal_gobal.std_echo_msg(0, "请先构建或双击激活一个已存在的工作空间(Projects下的项目文件)！")
                return 0

            if os.path.exists(stdinit.init_path):
                stdinit.pro_conf.clear()
                stdinit.pro_conf.read(stdinit.init_path, encoding="utf-8")  # python3
                it = []
                try:
                    ii = len(stdinit.pro_conf.sections())

                    if ii > 0:
                        for i in range(ii):
                            if "env:" in stdinit.pro_conf.sections()[i]:
                                it.append(stdinit.pro_conf.sections()[i].replace("env:", ""))

                        if len(it) > 1:

                            stdinit.std_signal_gobal.std_echo_msg(0, "当前工作空间platformio.ini文件中存在多款芯片，请注释掉其他芯片后再进行Debug操作！")
                            return 0
                        else:

                            _thread.start_new_thread(self.piodebug, ("Thread-1", 1,))
                            self.pio_de_start()
                            stdinit.std_signal_gobal.std_process(1, "DEBUG初始化")
                    else:

                        _thread.start_new_thread(self.piodebug, ("Thread-1", 1,))
                        self.pio_de_start()
                        stdinit.std_signal_gobal.std_process(1, "DEBUG初始化")
                except:
                    stdinit.std_signal_gobal.stdprintln()
                    return 0
            else:
                stdinit.std_signal_gobal.std_echo_msg(0,"请先构建或双击激活一个已存在的工作空间(Projects下的项目文件)！")


        except:
            stdinit.std_signal_gobal.stdprintln()



    def st_debug1(self):
        try:
            global debug_staus2
            global debug_staus
            global debug_staus_st

            if debug_staus2 == 1:

                try:
                    _thread.start_new_thread(self.startgdbserver, ("Thread-1", 1,))
                    # s=self.startgdbserver("Thread-1", 1)
                    time.sleep(1)
                    # _thread.start_new_thread(self.startgdb, ("Thread-2", 2,))
                    # time.sleep(1)
                    # self.gdb_init()
                    # s=1
                    # self.debug_sig.emit(0)
                    if debug_staus == 1:
                        pass
                    else:
                        if debug_staus_st == 1:
                            self.debug_sig.emit(0)
                            _thread.start_new_thread(self.startgdb, ("Thread-2", 2,))
                            time.sleep(1)
                            self.gdb_init()
                            # self.startgdb("Thread-2", 4)
                            self.debug_step_out.setDisabled(False)
                            self.debug_step_into.setDisabled(False)
                            self.debug_step_over.setDisabled(False)
                            self.de_continue.setDisabled(False)
                            self.restart.setDisabled(False)
                            self.de_stop.setDisabled(False)
                            self.de_start.setDisabled(True)
                            pass
                        else:
                            self.de_stop2()
                            QMessageBox.warning(self, "Can't DEBUG",
                                                "ST-LINK: Could not verify ST device! Abort connection",
                                                QMessageBox.Yes)
                            pass

                    # time.sleep(1)

                except:
                    # print("Error: 无法启动线程")
                    stdinit.std_signal_gobal.stdprintln()
            else:
                QMessageBox.warning(self, "Can't DEBUG", "Please recompile with the optimition of DEBUG mode First!",
                                    QMessageBox.Yes)
        except:
            stdinit.std_signal_gobal.stdprintln()





# if __name__ == "__main__":
#     import sys
#
#     app = QtWidgets.QApplication(sys.argv)
#     widget =StDebug()
#
#     widget.show()
#     sys.exit(app.exec_())


from cores import stdvarinit
from cores import fs
import urllib.request
class PlatformPackageManager(object):



    def install(self,spec,silent=False,force=False,):
        pa = stdvarinit.stdcache_dir + "/" + spec
        pas = stdvarinit.std_platform_dir + "/" + spec
        ud = "https://stduino-generic.pkg.coding.net/stduino_packages/stdplatforms/"+spec+".tar.gz?version=latest"

        urllib.request.urlretrieve(ud, pa)
        fs.untar_file(pa, pas)
        return True

    def uninstall(self):
        pass
    def update(self):
        pass

    pass



from cores import stdvarinit
import os


import urllib.request
import tarfile
import requests


def tools_is_on_coding(self, name, file_url):
    try:
        file_url = "https://dl.bintray.com/platformio/tool-packages/" + file_url
        url = "https://stduino-generic.pkg.coding.net/stduino_packages/piopackages/" + name[8:]
        html = requests.head(url)  # 用head方法去请求资源头
        re = html.status_code

        if re == 200:

            mirror_file_size_str = html.headers['Content-Length']  # 提取出来的是个数字str

    except:
        return False
def untar_file(self, file, tarpath):
    t = tarfile.open(file)
    t.extractall(path=tarpath)
    pass

def down_test():
    pa = stdvarinit.stdcache_dir + "/stdplatforms.json"
    size=os.path.getsize(pa)
    print(size)
    print(pa)
    pas = stdvarinit.std_platform_dir + "/"# + self.app.pargs.name
    print(pas)
    ud="https://stduino-generic.pkg.coding.net/stduino_packages/stdplatforms/stdplatforms.json?version=latest"

    html = requests.head(ud)  # 用head方法去请求资源头
    re = html.status_code

    if re == 200:
        mirror_file_size_str = html.headers['Content-Length']  # 提取出来的是个数字str
        print(mirror_file_size_str)

    #urllib.request.urlretrieve(ud, pa)

# class StdDownload(Controller):
#     class Meta:
#         label = 'stddownload'
#         stacked_type = 'nested'
#         stacked_on = 'base'
#         arguments = [
#             (['-t'], {'help': 'type of package file', 'action': 'store', 'dest': 'type'}),
#             (['-n'], {'help': 'name of package file', 'action': 'store', 'dest': 'name'}),
#         ]
#     plat=None
#
#     @ex(hide=True)
#     def _default(self):
#         self.downfile()
#     def geturl(self):
#         if self.app.pargs.type=="platform":
#             print("plat")
#             return "https://stduino-generic.pkg.coding.net/stduino_packages/stdplatforms/" + self.app.pargs.name
#
#         else:
#             return "https://stduino-generic.pkg.coding.net/stduino_packages/stdpackages/" + self.app.pargs.name
#
#         pass
#     def downfile(self):
#
#         pa=stdvarinit.stdcache_dir+"/"+ self.app.pargs.name
#         pas = stdvarinit.std_platform_dir + "/" + self.app.pargs.name
#         ud="https://stduino-generic.pkg.coding.net/stduino_packages/piopackages/framework-lgt8fx-1.0.6.tar.gz?version=latest"
#         url=self.geturl()
#         urllib.request.urlretrieve(ud, pa)
#         self.untar_file(pa, pas)
#
#         # with zipfile.ZipFile(pa, mode="r") as f:
#         #     f.extractall(pas)
#
#     def untar_file(self,file,tarpath):
#         t = tarfile.open(file)
#         t.extractall(path=tarpath)
#         pass
#     def tar_file(self,tarfile,tarpath):
#         pass
#     def delfile(self):
#         pass


if __name__ == '__main__':
    down_test()




# -*- coding: utf-8 -*-

"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""

#global var
from function.cores.stdedit.stdsignals import StdSinal
import os
from shutil import copy2
from function.conf import configparser3
from platform import system
platform_is=None
stdenv = None
projects_dir=None
if system() == "Windows" or system() == "cli":
    platform_is="Win"
    stdenv = os.environ["USERPROFILE"].replace('\\', '/')

    projects_dir = stdenv + "/Documents/Stduino/Projects/"
    username = os.environ['USERNAME']
elif system() == "Linux":
    platform_is="Linux"
    stdenv = os.environ["HOME"].replace('\\', '/')
    projects_dir = stdenv + "/Stduino/Projects/"
    username = os.environ['USERNAME']
    # ctags_path = stdinit.abs_path + "/tool/packages/ctagslin32/"
elif system() == "Darwin":
    platform_is = "Darwin"
    stdenv = os.environ["HOME"].replace('\\', '/')
    projects_dir = stdenv + "/Documents/Stduino/Projects/"
    username = os.environ['USER']
abs_path = os.path.abspath('.').replace('\\', '/')




#print(os.environ.keys())

# patherror = ''
# for char in stdenv:
#     if u'\u4e00' <= char <= u'\u9fa5':  # 判断是否是汉字，在isalpha()方法之前判断
#         patherror = patherror + char
# if patherror!="":
#     stdenv = "C:/Stduino"
#     if os.path.exists(stdenv):
#         pass
#     else:
#         os.makedirs(stdenv)

try:
    if os.path.exists(projects_dir):
        pass
    else:
        os.makedirs(projects_dir)

except:
    pass
std_platform_dir=stdenv + "/.stduino/platforms"
try:
    if os.path.exists(std_platform_dir):
        pass
    else:
        os.makedirs(std_platform_dir)
except:
    pass
session_dir=stdenv + "/.stduino/session"
try:
    if os.path.exists(session_dir):
        pass
    else:
        os.makedirs(session_dir)
        copy2(abs_path + "/appearance/config.cfg", stdenv + "/.stduino/session/config.cfg")
        copy2(abs_path + "/appearance/mainconfig.cfg", stdenv + "/.stduino/session/mainconfig.cfg")
        copy2(abs_path + "/tool/packages/other/comment", stdenv + "/.stduino/session/comment")
except:
    pass
#edit
auto_key_l=0
auto_key_list=[]
#IDE
version_c = "1.10.10"#'1.01'

#ui
savefileAction_staus=False
saveallAction_staus=False
forwardAction_staus=False
backAction_staus=False

#signal
std_signal_gobal=StdSinal()
thread_callch=0
#sub ui
File_tree_view=None
Std_Serial_Tool1= None
Std_Serial_Tool2= None
Set_Back_Color1=None
Std_makeboards=None
Std_Make=None
Std_Find=None
find_boards=None

device=None
std_make_board=None
#project
pio_boards=None
board_id=""
current_board=None
current_upload_m=None
upload_meds = []
profile_modify_ui=[]
profile_modify_all=[]
project_name=None#new_workspace=stdinit.pro_dir_name
pro_dir_name=None#old_workfpace=stdinit.pro_dir_name

#debug&goto

serialnumber=[]
last_com="None"
debug_mark_start = 0
manual_mark=[]
mark_object=[]
framework=None
platform=None
run_platforms=None
stduino_platforms=None
goto_platform_name=None
goto_init=None#.Instance
pro_conf=configparser3.ConfigParser()
init_path = None#platformio.ini
#plug
Pio_install=None
Piopalt_install=None
package_choose_name=""
plat_choose_name=""
#相关主目录
Std_help=None
Std_test="one"










import os
import time
from cores import stdvarinit
import click
from multiprocessing import cpu_count
import subprocess
try:
    DEFAULT_JOB_NUMS = cpu_count()
except NotImplementedError:
    DEFAULT_JOB_NUMS = 1

@stdvarinit.cli.group("run", short_help="run manager")
def cli():
    pass

@cli.command("build", short_help="Run project targets (build, upload, clean, etc.)")
@click.option("-e", "--environment", multiple=True)
@click.option("-t", "--target", multiple=True)
@click.option("--upload-port")
@click.option(
    "-d",
    "--project-dir",
    default=os.getcwd,
    type=click.Path(
        exists=True, file_okay=True, dir_okay=True, writable=True, resolve_path=True
    ),
)
@click.option(
    "-c",
    "--project-conf",
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, readable=True, resolve_path=True
    ),
)
@click.option(
    "-j",
    "--jobs",
    type=int,
    default=DEFAULT_JOB_NUMS,
    help=(
        "Allow N jobs at once. "
        "Default is a number of CPUs in a system (N=%d)" % DEFAULT_JOB_NUMS
    ),
)
@click.option("-s", "--silent", is_flag=True)
@click.option("-v", "--verbose", is_flag=True)
@click.option("--disable-auto-clean", is_flag=True)
@click.option("--list-targets", is_flag=True)
@click.pass_context
def build(
    ctx,
    environment,
    target,
    upload_port,
    project_dir,
    project_conf,
    jobs,
    silent,
    verbose,
    disable_auto_clean,
    list_targets,
):

    t1 = time.time()
    print(ctx,
    environment,
    target,
    upload_port,
    project_dir,
    project_conf,
    jobs,
    silent,
    verbose,
    disable_auto_clean,
    list_targets)
    os.chdir("C:/Users/debug/.stduino/packages/framework-arduino-w80x/tools/stduino/")  # 通过更改当前运行目录
    cmd = "C:/Users/debug/.stduino/packages/python3/python C:/Users/debug/.stduino/packages/framework-arduino-w80x/tools/stduino/test.py"
    cmd = "scons -Q -j 8"
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
    for line in iter(proc.stdout.readline, b''):
        # s1 = str(line)#
        #line=str(line).replace("\\r", "").replace("\\n", "").replace("\\n", "")
        try:

            s1 = str(line, encoding='gbk')
        except:
            s1 = str(line)
        s1=s1.strip()
        print(s1)#
        # response = gdbmiparser.parse_response(s1)
        # print(response)
        #click.echo(s1)
        # print(s1)
        if not subprocess.Popen.poll(proc) is None:
            if line == "":
                break
    proc.stdout.close()
    t2 = time.time()
    click.echo("========================= [SUCCESS] Took %s seconds =========================" % str(int(t2 - t1)))
    pass

    #app.set_session_var("custom_project_conf", project_conf)





#test
xyz=123

#menu

file='文件'
import_lib='导入库'
lib_example='示例'
editor='编辑'
project='项目'
tool='工具'
help='教程'

#menufile

openrecent='打开最近的'
projectfile='项目文件夹'
open_folder='打开项目所在文件夹'
exit='关闭软件'
commenter='注释/取消注释'
language='语言切换'
#toobar

complex='编译'
upload='编译及下载'
uploadfast='下载'
clean='清除'
new='新建'
open='打开'
save='保存'
saveall='全部保存'
serial='串口监视器'
serialp='串口绘图器'
search='立即搜索'
back='回退'
forward='向前'
editstyle='页面设置'
install_driver='手动安装驱动'
choose_down_type='选择下载方式'
usb_link='串口下载'
st_link='St_link'
tcommunity='技术社区'
community_web='社区官网'
about='关于'
li_development='库开发'
create_lib='新建库'
search_lib='搜索库'
open_lib='打开库所在文件夹'
choose_board='选择板型：'
choose_port='选择下载端口：'
check_port='检测端口'
restart_effect='重启设置生效'

lib_search='输入库名称'
lib_list='库列表'
board_list='新项目'
example_list='示例'
project_list='项目'
debug_button='调试'
platform_list='平台'
package_list='插件'
package_uninstall='卸载'
lib_detail='更多信息'
lib_select='选择要安装的文件'
lib_version='版本信息'
select_version='当前版本'
install_lib='确认安装'
lib_find_more='查找更多'
stduino_lib_install='Sduino 包安装'
no_net='无网络可访问，请连网后重试!'
format1='格式化'
paste='粘贴'
day_model='白天模式'
night_model='夜晚模式'
person_model='个性模式'
person_setting='个性化设置选项'
edit_back_color='编辑背景颜色'
main_back_color='正文背景颜色'
commenter_back_color='注释背景颜色'
confirm_change='确认修改'
concle_change='取消修改'
exit_this='退出'
find_next='查找/下一个'
replace_next='替换/下一个'
replace_all='全部替换'
match='完全匹配'
feedback='反馈'
serial_port='无可用串口!'
on=' 在 '
download_type_1=' 当前下载方式： '

complexing='正在编译：'
start_complex='开始编译...'
now_program='当前程序：'
row_line='行/列:'
warning_now_line='警告：行/列:'
complex_complete='编译完成!'
complex_failed='编译失败!'
clean_out='清除完成'
clean_failed='清除失败'
serial_port_check='请检查下载端口是否选择正确或请拔掉0号引脚上的连接线!'
serial_port_is_normal='检查串口是否正常！'
st_link_check='请检查St_link与单片机间是否正确连接或者请更新st_link固件!'
download_start='开始下载...'
downloading='正在下载：'
downloaded='下载完成!'
download_failed='下载失败!'
tip='提示'
lib_installed='文件包安装成功！'
lib_install_failed='安装失败，请稍后重试！'
lib_install_info='该库无可用版本，暂时无法下载！'
lib_download='正在下载：'
shouce='本地使用说明手册正在开发ing！'
server='我们将逐步完善为您提供更好的服务'
server_detail='问题描述\n意见建议\n您的邮箱'
feed_back='问题反馈！'
feed_back_thank='感谢您的反馈!您的意见将成为我们改进软件，提升服务的重要依据,欢迎您继续关注支持我们！'
feed_back_failed='反馈提交失败，请稍后再试'
about_ide='关于本软件'
function_development='本功能正在开发！'
lib_share='选择要分享的库'
language_choose='Stduino IDE 库开发语言选择'
language_select='请选择库开发语言'
cancel='取消'
lib_name='库文件名'
file_name="文件名:"
line_text='//One keyword per line, delete the line after completion'
save_tip='被改动是否保存'
save_tip_file='是否保存该文件'
last_close='最后一个窗口将关闭软件'
find_text_none='查找完毕:无此关键词!'
change_text_none='替换完毕:无此关键词!'
change_text_num='替换完毕:共替换'
mail_sended='邮件发送成功'
mail_send_failed='邮件发送失败,请稍后再试'
serial_port_helper='串口助手'
serial_open_failed='此串口不能被打开！'
serial_port_staus_o='串口状态（已开启）'
serial_port_staus_c='串口状态（已关闭）'
input_data_16='请输入十六进制数据，以空格分开!'
send_hex='发HEX['
send_dec='发DEC['
receive_hex='收HEX['
receive_dec='收DEC['
reset_up='重启软件后生效'
startup_failed='软件启动失败'
search_tip='请至官网搜索“软件无法启动解决措施”！'
web_site='官方网站：'
file_save='文件保存'
select_file='选取文件'

add_file="添加文件至项目"
add_direc="添加项目文件夹"
re_name="重命名"
delete_f_d="删除文件/夹"

code_flash="代码总量:"
code_ram=" 总内存:"
code_static=" 所占百分比:"
code_bits="字节"

project_set="新建项目配置"
project_name="项目名称"
board_core_type="芯片"
project_framework="框架"
board_usb="USB支持"
board_usb_speed="USB速度"
board_opt="编译设置"
board_runtime="运行依赖库"
board_upload="下载方式"
board_debug="调试方式"
project_build="创建"
library_installer="退出当前安装界面"
search_sucess="搜索成功"


# !/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
__author__ = 'Su Jin Qiang'

from .stdedit import stdinit

from PyQt5.QtWidgets import QHBoxLayout,QGridLayout,QPushButton,QCheckBox,QColorDialog,QFontDialog,QComboBox,QMessageBox,QLabel,QWidget

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIcon
import os
from function.conf import res, setup
from .stdmsg import reso


#import tempfile
# 开发本软件消耗了太多时间精力，一路走来，太不容易，写下此行留为纪念，尤其感谢几位学弟（林鉴波、梁莅、房杰、刘席鸣等一直以来的各方面支持）。——2019.10.22晚


my_sender = 'sam9661@qq.com'  # 发件人邮箱账号sam9661@qq.com  stduino@foxmail.com
my_pass = 'jrtuhtpsukpocjii'  # 发件人邮箱密码   squxhquxrlalbaae
my_user = 'service001@stduino.com'  # 收件人邮箱账号，我这边发送给自己

#pos = os.path.abspath('.')
#path = pos + "\\tool\msy"

#os.chdir("C:\stwork\stdemo2019827\dist\Stduinodebug\main")  # 通过更改当前运行目录F:\BaiduNetdiskDownload\stpython\stdemo\main

class StdBackColor(QWidget):
    def __init__(self):
        try:
            super(StdBackColor, self).__init__()
            self.setWindowOpacity(0.7)
            self.setGeometry(800, 200, 520, 280)
            # self.setFixedSize(420, 220)
            self.setWindowTitle('Background Setting')
            self.setWindowFlags(Qt.WindowStaysOnTopHint)
            self.setWindowIcon(QIcon("appearance/img/st.PNG"))
            self.init_path = stdinit.session_dir + "/mainconfig.cfg"

            # 全局布局（2中）：这里选择水平布局
            wlayout = QHBoxLayout()

            # 局部布局：水平，垂直，网格，表单

            glayout = QGridLayout()

            # 下拉框
            #
            self.combo = QComboBox(self)
            self.combo.addItem(reso.day_model)
            self.combo.addItem(reso.night_model)
            self.combo.addItem(reso.person_model)
            self.combo.setCurrentIndex(res.default_c)
            self.combo.currentIndexChanged.connect(self.toggle)

            # Qlabel

            lbl = QLabel(reso.person_setting, self)
            # lbl1 = QLabel("是否确认更改", self)

            # self.set1_button = QPushButton("字体更改")
            # 绑定传参方式
            # self.set1_button.clicked.connect(lambda: self.c_font(0))
            # 1.编辑栏背景 2.正常文字背景 python3.注释背景颜色 4.主关键字背景 5.函数关键字背景 6.括号背景
            self.set2_button = QPushButton(reso.edit_back_color)
            self.set2_button.clicked.connect(lambda: self.c_color(1))
            self.set3_button = QPushButton(reso.main_back_color)
            self.set3_button.clicked.connect(lambda: self.c_color(2))
            self.set4_button = QPushButton(reso.commenter_back_color)
            self.set4_button.clicked.connect(lambda: self.c_color(3))

            self.set15_button = QPushButton("主关键字背景")
            self.set15_button.clicked.connect(lambda: self.c_color(4))
            self.set16_button = QPushButton("函数关键字背景")
            self.set16_button.clicked.connect(lambda: self.c_color(5))
            self.set17_button = QPushButton("括号分号背景")
            self.set17_button.clicked.connect(lambda: self.c_color(6))
            self.set18_button = QPushButton("汉字背景")
            self.set18_button.clicked.connect(lambda: self.c_color(7))
            self.set19_button = QPushButton("数字背景")
            self.set19_button.clicked.connect(lambda: self.c_color(8))
            self.set20_button = QPushButton("预编译背景")
            self.set20_button.clicked.connect(lambda: self.c_color(9))
            self.stdauto_complete=QCheckBox("括号引号自动完成")
            self.stdauto_complete.clicked.connect(self.stdauto_action)
            self.stdauto_complete.setChecked(res.std_auto)

            self.set5_button = QPushButton(reso.confirm_change)
            self.set5_button.clicked.connect(self.confirm)
            self.set6_button = QPushButton(reso.concle_change)
            self.set6_button.clicked.connect(self.concel)
            self.set7_button = QPushButton(reso.exit_this)
            self.set7_button.clicked.connect(self.close)
            self.set6_button.setEnabled(False)
            self.set5_button.setEnabled(False)

            if res.default_c < 2:
                self.set2_button.setEnabled(False)
                self.set3_button.setEnabled(False)
                self.set4_button.setEnabled(False)
                self.set15_button.setEnabled(False)
                self.set16_button.setEnabled(False)
                self.set17_button.setEnabled(False)
                self.set18_button.setEnabled(False)
                self.set19_button.setEnabled(False)
                self.set20_button.setEnabled(False)

                pass
            else:
                self.set2_button.setEnabled(True)
                self.set3_button.setEnabled(True)
                self.set4_button.setEnabled(True)
                self.set15_button.setEnabled(True)
                self.set16_button.setEnabled(True)
                self.set17_button.setEnabled(True)
                self.set18_button.setEnabled(True)
                self.set19_button.setEnabled(True)
                self.set20_button.setEnabled(True)
                pass

            # line edit
            # LineEdit1 = QLineEdit()

            glayout.addWidget(lbl, 1, 0)
            # glayout.addWidget(LineEdit1, 1, 0)
            # glayout.addWidget(lbl1,1,1)

            # glayout.addWidget(lbl,0,1)
            glayout.addWidget(self.combo, 2, 0)
            # glayout.addWidget(QPushButton("确认"),2,1)
            # glayout.addWidget(QPushButton("取消"),2,2)

            glayout.addWidget(self.set2_button, 3, 0)
            glayout.addWidget(self.set3_button, 4, 0)

            glayout.addWidget(self.set4_button, 5, 0)
            glayout.addWidget(self.set18_button, 2, 1)
            glayout.addWidget(self.set15_button, 3, 1)
            glayout.addWidget(self.set16_button, 4, 1)

            glayout.addWidget(self.set17_button, 5, 1)
            glayout.addWidget(self.set19_button, 2, 2)
            glayout.addWidget(self.set20_button, 3, 2)
            glayout.addWidget(self.stdauto_complete, 4, 2)

            # glayout.addWidget("cd", 6, 0)

            glayout.addWidget(self.set5_button, 7, 1)
            glayout.addWidget(self.set6_button, 7, 2)
            glayout.addWidget(self.set7_button, 7, 3)
            # 准备四个控件
            gwg = QWidget()
            # 使用四个控件设置局部布局
            gwg.setLayout(glayout)
            # 将四个控件添加到全局布局中
            wlayout.addWidget(gwg)
            # 将窗口本身设置为全局布局
            self.setLayout(wlayout)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def stdauto_action(self):
        try:

            if os.path.exists(self.init_path):
                stdinit.pro_conf.clear()
                if res.std_auto==1:
                    res.std_auto=0
                else:
                    res.std_auto = 1
                stdinit.pro_conf.read(self.init_path, encoding="utf-8")  # python3
                stdinit.pro_conf.set("std_type", "std_auto", str(res.std_auto))
                stdinit.pro_conf.write(open(self.init_path, 'w'))

        except:
            stdinit.std_signal_gobal.stdprintln()




    def c_color(self, idd):
        try:
            self.set5_button.setEnabled(True)
            self.set6_button.setEnabled(True)
            # print(121)
            color = QColorDialog.getColor(Qt.blue, self, "Select Color")
            # 1.编辑栏背景 2.正常文字背景 python3.注释背景颜色 4.主关键字背景 5.函数关键字背景 6.括号背景
            if color.isValid():
                # return color.name()
                # the_window.setcolor_all(color.name(), idd)
                stdinit.std_signal_gobal.std_back_color(color.name(), idd)
                if idd == 1:
                    self.set2_button.setStyleSheet("background-color:" + color.name() + ";")  ########## 串口查找按钮
                elif idd == 2:
                    self.set3_button.setStyleSheet("background-color:" + color.name() + ";")  ########## 串口查找按钮
                elif idd == 3:
                    self.set4_button.setStyleSheet("background-color:" + color.name() + ";")  ########## 串口查找按钮
                elif idd == 4:
                    self.set15_button.setStyleSheet("background-color:" + color.name() + ";")  ########## 串口查找按钮
                elif idd == 5:
                    self.set16_button.setStyleSheet("background-color:" + color.name() + ";")  ########## 串口查找按钮
                elif idd == 6:
                    self.set17_button.setStyleSheet("background-color:" + color.name() + ";")  ########## 串口查找按钮
                elif idd == 7:
                    self.set18_button.setStyleSheet("background-color:" + color.name() + ";")  ########## 串口查找按钮
                elif idd == 8:
                    self.set19_button.setStyleSheet("background-color:" + color.name() + ";")  ########## 串口查找按钮
                else:
                    self.set20_button.setStyleSheet("background-color:" + color.name() + ";")  ########## 串口查找按钮
        except:
            stdinit.std_signal_gobal.stdprintln()



            # QsciLexerC.__lexer.setColor(QColor(color.name()), QsciLexerCPP.CommentLine)  # 注释 #66CDAA  #1C1C1C  黑
            # print(color.name(), color)
            # self.myButton.setPalette(QPalette(color))  # 给按钮填充背景色
            # self.myButton.setAutoFillBackground(True)

    def c_font(self, idd):
        try:
            font, ok = QFontDialog.getFont()
            if ok:
                # the_window.setcolor_all(font, idd)
                stdinit.std_signal_gobal.std_back_color(font, idd)
                # print(font)
        except:
            stdinit.std_signal_gobal.stdprintln()



    def confirm(self, event):
        try:
            self.set6_button.setEnabled(True)
            self.set5_button.setEnabled(False)

            reply = QMessageBox.question(self, reso.tip, reso.reset_up,
                                         QMessageBox.Yes)
            if reply == QMessageBox.Yes:
                setup.save_confirm(str(self.combo.currentIndex()))
                # event.accept()
            else:
                pass
        except:
            stdinit.std_signal_gobal.stdprintln()




    def concel(self):
        try:
            self.set6_button.setEnabled(False)
            setup.save_confirm(str(res.default_c))
        except:
            stdinit.std_signal_gobal.stdprintln()


        # setup.save_mstyle(res.default_main)
        # setup.save_mstyle(res.default_text)
        # setup.save_cstyle(res.default_comment)

    def toggle(self, state):
        try:
            self.set5_button.setEnabled(True)
            # setup.save_confirm(state)
            if state < 2:
                self.set2_button.setEnabled(False)
                self.set3_button.setEnabled(False)
                self.set4_button.setEnabled(False)
                self.set15_button.setEnabled(False)
                self.set16_button.setEnabled(False)
                self.set17_button.setEnabled(False)
                self.set18_button.setEnabled(False)
                self.set19_button.setEnabled(False)
                self.set20_button.setEnabled(False)
                pass
            else:
                self.set2_button.setEnabled(True)
                self.set3_button.setEnabled(True)
                self.set4_button.setEnabled(True)
                self.set15_button.setEnabled(True)
                self.set16_button.setEnabled(True)
                self.set17_button.setEnabled(True)
                self.set18_button.setEnabled(True)
                self.set19_button.setEnabled(True)
                self.set20_button.setEnabled(True)
                pass
        except:
            stdinit.std_signal_gobal.stdprintln()





# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
# !/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
__author__ = 'Su Jin Qiang'


import os

from PyQt5.QtWidgets import QListView

from PyQt5.QtWidgets import QWidget,QLabel,QGridLayout,QAbstractItemView,QFileDialog


from . import stdinit
from PyQt5.QtCore import QStringListModel

# 开发本软件消耗了太多时间精力，一路走来，太不容易，写下此行留为纪念，尤其感谢几位学弟（林鉴波、梁莅、房杰、刘席鸣等一直以来的各方面支持）。——2019.10.22晚

#pos = os.path.abspath('.')
#path = pos + "\\tool\msy"

#os.chdir("C:\stwork\stdemo2019827\dist\Stduinodebug\main")  # 通过更改当前运行目录F:\BaiduNetdiskDownload\stpython\stdemo\main



class StdFilesView(QWidget):

    # Create a Json file for a better path management

    def __init__(self):
        try:

            # self.staus_change.connect(self.staus_change.emit)
            # if

            super(StdFilesView, self).__init__()
            # self.setWindowOpacity(0.7)
            # self.setGeometry(50, 50, 50, 30)
            # self.my_signal_findboard.connect(self.sig_find_boards)
            # self.setFixedSize(420, 220)
            # self.setWindowTitle('New Project')
            # 置顶及去标题栏

            #self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint)
            # self.setWindowIcon(QIcon("appearance/img/st.PNG"))
           # self.setStyleSheet('background: DimGrey')

            self.start_up_label = QLabel(self)
            self.start_up_label.setStyleSheet('font-size:25px;color:Ivory')
            # self.begin_label.setGeometry(QtCore.QRect(20, 5, 225, 22))
            self.start_up_label.setText("启动")
            self.start_up_label.setObjectName("start_up")
            self.start_up_buttons = QListView(self)
            self.start_up_buttons.setMaximumWidth(400)
            self.start_up_buttons.setStyleSheet('border:none;text-align:left;font-size:20px;color:MediumBlue')
            self.start_up_buttons_model = QStringListModel(self)
            self.start_up_buttons.setEditTriggers(QAbstractItemView.NoEditTriggers)

            self.start_up_buttons_model_list = ["新建工程...","打开工程...","打开使用帮助...","主题设置..."]

            self.start_up_buttons_model.setStringList(self.start_up_buttons_model_list)
            self.start_up_buttons.setModel(self.start_up_buttons_model)
            self.start_up_buttons.clicked.connect(self.start_buttons_clicked)


            self.recently_files_label = QLabel(self)


            self.recently_files_label.setMinimumWidth(260)
            # self.board_label.setGeometry(QtCore.QRect(20, 40, 225, 22))
            self.recently_files_label.setText("最近")
            self.recently_files_label.setStyleSheet('font-size:25px;color:Ivory')

            self.recently_files_label.setObjectName("recently_files_label")
            self.projects_path_model_list=[]

            # self.pio_pro_boards = self.pio_pro.project_boards()

            # wlayout = QVBoxLayout()
            self.projects_path=QListView(self)
            self.projects_path.setEditTriggers(QAbstractItemView.NoEditTriggers)

            self.projects_path.setMaximumWidth(560)
            self.projects_path.setStyleSheet('border:none;text-align:left;font-size:20px;color:MediumBlue')
            self.projects_path_model = QStringListModel(self)



            self.recently_files_path_list()
            self.projects_path_model.setStringList(self.projects_path_model_list)
            self.projects_path.setModel(self.projects_path_model)
            self.projects_path.clicked.connect(self.open_recently_project_click)

            # 局部布局：水平，垂直，网格，表单
            glayout = QGridLayout()
            # line edit
            # LineEdit1 = QLineEdit()

            glayout.setSpacing(10)
            glayout.addWidget(self.start_up_label,0, 1)
            glayout.addWidget(self.start_up_buttons, 1, 1)



            glayout.addWidget(self.recently_files_label, 0,2)  # name platform board fromwork  下载方式

            glayout.addWidget(self.projects_path, 1, 2)





            #glayout.setRowStretch(0, 1)
            # 准备四个控件
            # gwg = QWidget()
            # # 使用四个控件设置局部布局
            # gwg.setLayout(glayout)
            # # 将四个控件添加到全局布局中
            # wlayout.addWidget(gwg)
            self.setLayout(glayout)
        except:
            print(232)
            # stdinit.std_signal_gobal.stdprintln()
    def show2(self):
        self.show()
    def recently_files_path_list(self):
        for i in stdinit.File_tree_view.ProjectXmltreeRoot:

            self.projects_path_model_list.append(i.attrib['projectpath'])
    def open_recently_project_click(self,qModelIndex):
        action_num = qModelIndex.row()
        stdinit.std_signal_gobal.std_project_path_observer(self.projects_path_model_list[action_num])
        stdinit.std_signal_gobal.std_main_ui_setview(0)
        pass



    def start_buttons_clicked(self,qModelIndex):
        action_num = qModelIndex.row()
        if action_num==0:
            stdinit.find_boards.load_boards()
            stdinit.find_boards.show()
        elif action_num==1:
            m = QFileDialog.getExistingDirectory(None, "选取工程文件夹", stdinit.projects_dir)  # 起始路径
            if os.path.exists(m):
                stdinit.std_signal_gobal.std_project_path_observer(m)
                stdinit.std_signal_gobal.std_main_ui_setview(0)

            pass
        elif action_num==2:
            stdinit.std_signal_gobal.std_echo_msg(1,"先手动打开wiki.stduino.com吧，后面我再继续优化这个功能")

        elif action_num==3:
            stdinit.std_signal_gobal.std_echo_msg(1,"抱歉哈，您先再等等，我还在继续搞这块的功能，待更新")

        pass



# if __name__ == '__main__':  # main函数
#     app = QApplication(sys.argv)
#     try:
#
#
#         find_boards = StdFilesView()
#         find_boards.show()
#
#
#     except:
#
#         sys.exit(app.exec_())
#
#     sys.exit(app.exec_())




# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""
#
# from six.moves import urllib  # 下载文件
import json
#import zipfile  # 解压文件
# import shutil  # 用于压缩
# import subprocess
import os,stat
import sys
# import re
# import socket
#
# import zipfile
#
# import time

from PyQt5.QtWidgets import QComboBox,QTextEdit,QLabel,QVBoxLayout,QWidget,QHBoxLayout,QPushButton,QGridLayout,QGroupBox,QLineEdit,QCheckBox,QListView,QAbstractItemView

from PyQt5.QtCore import Qt,QStringListModel,pyqtSignal
from ..piobuilder.piolibinstall import PioLibInstall

# from function.conf import res
from ..cores.stdmsg import reso
from ..cores.stdedit import stdinit

import threading



# fo = open("./appearance/stduino_json/library.json", mode='r', encoding='UTF-8')
# st = fo.read()
# fo.close()
# libraries = json.loads(st)
class StdAddLibs(QWidget):  # 安装库文件

    add_lib_s = pyqtSignal(int,int,str)

    def __init__(self):

        super(StdAddLibs, self).__init__()
        try:
            self.lib = PioLibInstall()
            self.add_lib_s_load_f = 0
            self.lib_p = 1
            self.find_more_p = 1
            self.pio_libs = []
            self.pio_lib=[]
            self.pio_libs.append(0)
            self.search_more_p = 1
            self.lib_list = []
            self.choose_id = 0
            self.initUi()
        except:
            stdinit.std_signal_gobal.stdprintln()



    def initUi(self):
        try:
            self.bigEditor = QTextEdit()
            self.bigEditor.setReadOnly(True)

            self.createGridGroupBox()
            self.createGridGroupBox2()
            self.creatVboxGroupBox()
            self.creatVboxGroupBox2()
            mainLayout = QVBoxLayout()
            hboxLayout = QHBoxLayout()
            hboxLayout.addWidget(self.vboxGroupBox2)
            hboxLayout.addWidget(self.vboxGroupBox)

            self.find_more = QPushButton(reso.lib_find_more, self)
            self.find_more.clicked.connect(self.find_mores)
            self.install_return = QPushButton(reso.library_installer, self)
            self.install_return.clicked.connect(self.install_ret)
            self.install = QPushButton(reso.install_lib, self)
            self.install.setDisabled(True)
            self.install.clicked.connect(self.install_url)


            glayout = QGridLayout()
            glayout.addWidget(self.install, 0, 0)
            glayout.addWidget(self.find_more, 0, 1)
            glayout.addWidget(self.install_return, 0, 2)


            # 准备四个控件
            gwg = QWidget()
            # 使用四个控件设置局部布局
            gwg.setLayout(glayout)
            # 将四个控件添加到全局布局中


            mainLayout.addWidget(self.gridGroupBox)
            mainLayout.addLayout(hboxLayout)
            mainLayout.addWidget(self.gridGroupBox2)
            #mainLayout.addWidget(self.pro1)
            mainLayout.addWidget(gwg)
            #mainLayout.addWidget(self.install_return)
            self.setLayout(mainLayout)
        except:
            stdinit.std_signal_gobal.stdprintln()
    def check_net(self):
        try:
            return stdinit.std_signal_gobal.is_connected()

        except:
            return False
    def libr_load(self):
        try:
            if self.check_net()==False:
                return False
            if self.add_lib_s_load_f == 0:
                self.add_lib_s_load_f = 1
                t1 = threading.Thread(target=self.library_load, name='li_load1')
                t1.setDaemon(True)
                t1.start()

        except:
            stdinit.std_signal_gobal.stdprintln()


    def createGridGroupBox(self):
        try:
            self.gridGroupBox = QGroupBox(reso.search_lib)
            layout = QGridLayout()
            nameLabel = QLabel(reso.lib_search)
            self.nameLineEdit = QLineEdit("")
            # self.nameLineEdit.editingFinished.connect(self.find_lib)
            #self.nameLineEdit.textChanged.connect(self.find_lib)
            self.search_lib_button=QPushButton(reso.search)
            self.search_lib_button.clicked.connect(self.find_lib)
            layout.setSpacing(10)
            layout.addWidget(nameLabel, 1, 0)
            layout.addWidget(self.nameLineEdit, 1, 1)
            layout.addWidget(self.search_lib_button, 1,2)
            layout.setColumnStretch(1, 10)
            self.gridGroupBox.setLayout(layout)
            self.setWindowTitle(reso.stduino_lib_install)
            #self.setFixedWidth(955)
            #self.setWindowIcon(QIcon("appearance/img/st.PNG"))
            #self.setFixedHeight(684)
            self.setWindowFlags(Qt.WindowStaysOnTopHint)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def createGridGroupBox2(self):
        try:
            self.gridGroupBox2 = QGroupBox(reso.lib_version)
            self.layout = QGridLayout()
            # nameLabel = QLabel("发布版本时间")
            self.vsion = QCheckBox(reso.select_version, self)
            self.vsion.setChecked(True)
            self.vsion.setEnabled(False)
            #self.vsion.stateChanged.connect(self.check_vsion)
            self.combobox_1 = QComboBox(self)
            self.combobox_1.setDisabled(True)
            self.layout.setSpacing(10)
            self.layout.addWidget(self.vsion, 1, 0)
            self.combobox_1.addItem("")
            self.layout.addWidget(self.combobox_1, 1, 1)
            self.layout.setColumnStretch(1, 10)
            self.gridGroupBox2.setLayout(self.layout)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def creatVboxGroupBox(self):
        try:
            self.vboxGroupBox = QGroupBox(reso.lib_detail)
            layout = QVBoxLayout()
            self.bigEditor.setPlainText(reso.lib_select)
            layout.addWidget(self.bigEditor)
            self.vboxGroupBox.setLayout(layout)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def creatVboxGroupBox2(self):

        try:
            self.vboxGroupBox2 = QGroupBox(reso.lib_list)
            layout = QVBoxLayout()

            self.model_1 = QStringListModel(self)
            self.listview_1 = QListView(self)  # python3
            self.lib_list = ["正在加载中~"]
            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)

            self.listview_1.setEditTriggers(QAbstractItemView.NoEditTriggers)
            #self.listview_1.setFixedWidth(600)
            #self.listview_1.setFixedHeight(400)
            self.listview_1.clicked.connect(self.list_clicked)
            layout.addWidget(self.listview_1)
            self.vboxGroupBox2.setLayout(layout)
                    # self.listview_1.doubleClicked.connect(lambda: self.change_func(self.listview_1))
                    # self.listview_1.doubleClicked.connect(self.cs)

                    #self.listview_1.clicked.connect(self.clicked)
        except:
            self.lib_list = [reso.no_net]
            self.model_1 = QStringListModel(self)
            self.listview_1 = QListView(self)  # python3
            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)
            self.listview_1.setEditTriggers(QAbstractItemView.NoEditTriggers)
            #self.listview_1.setFixedWidth(600)
            #self.listview_1.setFixedHeight(400)

            # self.item_list = ['item %s' % i for i in range(51)]  # 1
            stdinit.std_signal_gobal.stdprintln()

    def library_load(self):
        try:
            if self.check_net()==False:
                return False

            self.lib_list.clear()
            self.pio_lib = self.lib.lib_search(self.find_more_p)
            if self.pio_lib == None:

                target = stdinit.stdenv + "/.stduino/packages/pioenv/pioboards.json"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
                if os.path.exists(target):
                    stdinit.std_signal_gobal.std_echo_msg(0, "请重启软件再次打开该界面若还存在该问题，请至插件安装界面卸载并重新安装Platformio！")
                else:
                    stdinit.std_signal_gobal.std_echo_msg(0, "当前插件安装存在文件缺失，请至插件安装界面卸载并重新安装Platformio！")
                return 0
            stdinit.std_signal_gobal.std_echo_msg(1, "Page:" + str(self.pio_lib['page']) + " | Total:" + str(
                self.pio_lib['total']))
            #self.pio_libs.append(self.pio_lib)

            ii = len(self.pio_lib['items'])
            if ii > 0:
                pass
            else:
                return 0
            i = 0
            while i < ii:
                try:  # libs['items'][0]['name']if _reldir else src_dir
                    try:
                        down="1000"
                        down = str(self.pio_lib['items'][i]['dllifetime'])
                        worker=self.pio_lib['items'][i]['authornames'][0]
                    except:

                        worker="unknown"
                    lib_show =self.pio_lib['items'][i]['name'] + "\nby " + \
                                worker+ "    |    Downloads:" + down +"\nSupported frameworks: "
                    i_lib = len(self.pio_lib['items'][i]['frameworks'])
                    i_li = 0
                    while i_li < i_lib:
                        if i_li > 0:
                            lib_show = lib_show + "," + self.pio_lib['items'][i]['frameworks'][i_li][
                                "title"]
                        else:
                            lib_show = lib_show + self.pio_lib['items'][i]['frameworks'][i_li][
                                "title"]
                        i_li = i_li + 1
                    self.lib_list.append(lib_show + "\n")
                except:
                    stdinit.std_signal_gobal.stdprintln()
                    break
                i = i + 1
            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)

        except:
            stdinit.std_signal_gobal.stdprintln()




    def install_ret(self):
        try:

            #self.libui_change.emit()
            stdinit.std_signal_gobal.std_main_ui_change(1)
            # the_window.m_stackedLayout.setCurrentIndex(0)
            # the_window.liAct2.setEnabled(True)

        except:
            stdinit.std_signal_gobal.stdprintln()
    def list_clicked(self, qModelIndex):
        try:

            n_lin=qModelIndex.row()
            #n_lib = int(n_lin / 10)+1
            # if n_lin>9:
            #     n_lin = int(n_lin % 10)
            descrabe = self.pio_lib["items"][n_lin]['description']+ "\n\nTag:"
            i_lib = len(self.pio_lib['items'][n_lin]['keywords'])
            i_li = 0
            while i_li < i_lib:
                if i_li > 0:
                    descrabe = descrabe + "," + self.pio_lib['items'][n_lin]['keywords'][i_li]
                else:
                    descrabe = descrabe + self.pio_lib['items'][n_lin]['keywords'][i_li]

                i_li = i_li + 1
            descrabe=descrabe+"\n\nSupported platforms:"
            i_lib = len(self.pio_lib['items'][n_lin]['platforms'])
            i_li = 0
            while i_li < i_lib:
                if i_li > 0:
                    descrabe = descrabe + "," + self.pio_lib['items'][n_lin]['platforms'][i_li]['title']
                else:
                    descrabe = descrabe + self.pio_lib['items'][n_lin]['platforms'][i_li]['title']

                i_li = i_li + 1

            # for i in

            self.combobox_1.clear()
            #self.pio_libs[self.find_more_p]['items'][i]['name']

            self.choose_id=self.pio_lib['items'][n_lin]['id']

            self.combobox_1.addItem(self.pio_lib["items"][n_lin]['updated'])

            if self.install.isEnabled() == False:
                self.install.setDisabled(False)

            # if self.find_more.isEnabled() == False:
            #     self.find_more.setDisabled(False)

            self.bigEditor.setText(descrabe)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def remove_readonly(self,func, path, _):
        try:
            #"Clear the readonly bit and reattempt the removal"
            os.chmod(path, stat.S_IWRITE)
            func(path)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def install_lib(self):

        try:
            if self.check_net()==False:
                return False
            stdinit.std_signal_gobal.std_process(1,reso.downloading)
            self.lib.lib_install(str(self.choose_id))
            self.install.setDisabled(False)
            self.listview_1.setEnabled(True)
            stdinit.std_signal_gobal.std_process(0, reso.downloading)
        except:
            stdinit.std_signal_gobal.stdprintln()
            stdinit.std_signal_gobal.std_process(0, reso.downloading)

    def install_url(self):
        # from multiprocessing import Process
        # p = Process(target=self.install_url1, args=(1,))
        # print(1)
        try:
            if self.check_net()==False:
                return False
            self.install.setEnabled(False)
            self.listview_1.setEnabled(False)
            #self.my_thread = stdthread()  # 步骤2. 主线程连接子线,同时传递一个值给子线程
            # self.my_thread.std_signal.connect(self.thread_return)
            # stdinit.callch = 1
            # self.my_thread.start()  # 步骤3 子线程开始执行run函数
            t1 = threading.Thread(target=self.install_lib, name='th1', args=())
            t1.setDaemon(True)
            t1.start()

        except:
            stdinit.std_signal_gobal.stdprintln()
        #t1 = threading.Thread(target=pay_ticket, name='th1')



    def schedule(self,blocknum, blocksize, totalsize):
        try:
            # """
            #         blocknum:当前已经下载的块
            #         blocksize:每次传输的块大小
            #         totalsize:网页文件总大小
            #         """
            per = 100 * blocknum * blocksize / totalsize

            if per > 100:
                per = 100
                self.install_return.setText("Installing Library...")

            self.add_lib_s.emit(2, int(per), "")  # 2进度条数值

        except:
            stdinit.std_signal_gobal.stdprintln()


    def find_more_lib(self):
        try:
            if self.check_net()==False:
                return False
            #self.install_return.setText("Finding Library File...")
            stdinit.std_signal_gobal.std_process(1,reso.search)
            self.find_more.setEnabled(False)
            self.find_more_p = self.find_more_p + 1
            self.lib_p= self.lib_p + 1
            self.lib_list.clear()
            self.pio_lib=self.lib.lib_search(self.find_more_p)
            if self.pio_lib == None:

                target = stdinit.stdenv + "/.stduino/packages/pioenv/pioboards.json"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
                if os.path.exists(target):
                    stdinit.std_signal_gobal.std_echo_msg(0, "请重启软件再次打开该界面若还存在该问题，请至插件安装界面卸载并重新安装Platformio！")
                else:
                    stdinit.std_signal_gobal.std_echo_msg(0, "当前插件安装存在文件缺失，请至插件安装界面卸载并重新安装Platformio！")
                return 0
            #self.pio_libs.append(self.pio_lib)
            stdinit.std_signal_gobal.std_echo_msg(1, "Page:" + str(self.pio_lib['page']) + " | Total:" + str(self.pio_lib['total']))
            ii = len(self.pio_lib['items'])
            i = 0
            while i < ii:
                self.lib_list.append(self.pio_lib['items'][i]['name'] + "\ndescription:" +
                                     self.pio_lib['items'][i]['description'] + "\n")  #
                i = i + 1
            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)
            self.find_more.setEnabled(True)
            stdinit.std_signal_gobal.std_process(0, reso.search)
            stdinit.std_signal_gobal.std_echo_msg(1,reso.search_sucess)
        except:
            self.find_more.setEnabled(True)
            stdinit.std_signal_gobal.std_process(0, reso.search)
            stdinit.std_signal_gobal.std_echo_msg(1, reso.download_failed)
            stdinit.std_signal_gobal.stdprintln()
            pass
    def find_mores(self):
        try:

            t1 = threading.Thread(target=self.find_more_lib, name='th1', args=())
            t1.setDaemon(True)
            t1.start()
        except:
            stdinit.std_signal_gobal.stdprintln()



    def check_vsion(self):
        try:
            if self.vsion.isChecked():
                self.combobox_1.setDisabled(False)
            else:
                self.combobox_1.setDisabled(True)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def search_load(self):
        try:
            if self.check_net()==False:
                return False
            self.lib_list.clear()
            self.pio_lib = self.lib.lib_search_k(self.nameLineEdit.text(),self.search_more_p)
            if self.pio_lib==None:

                target = stdinit.stdenv + "/.stduino/packages/pioenv/pioboards.json"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
                if os.path.exists(target):
                    stdinit.std_signal_gobal.std_echo_msg(0, "请重启软件再次打开该界面若还存在该问题，请至插件安装界面卸载并重新安装Platformio！")
                else:
                    stdinit.std_signal_gobal.std_echo_msg(0, "当前插件安装存在文件缺失，请至插件安装界面卸载并重新安装Platformio！")
                return 0

            stdinit.std_signal_gobal.std_echo_msg(1, "Page:" + str(self.pio_lib['page']) + " | Total:" + str(self.pio_lib['total']))


            self.lib_p=self.lib_p+1
            self.search_more_p=self.search_more_p+1
            #self.pio_libs.append(self.pio_lib)
            try:

                ii = len(self.pio_lib['items'])
                if ii>0:
                    pass
                else:
                    return 0
                i = 0
                while i < ii:
                    try:  # libs['items'][0]['name']
                        lib_show = self.pio_lib['items'][i]['name'] + "\nby " + \
                                   self.pio_lib['items'][i]['authornames'][0] + "    |    Downloads:" + str(
                            self.pio_lib['items'][i]['dllifetime']) + "\nSupported frameworks: "
                        i_lib = len(self.pio_lib['items'][i]['frameworks'])
                        i_li = 0
                        while i_li < i_lib:
                            if i_li > 0:
                                lib_show = lib_show + "," + \
                                           self.pio_lib['items'][i]['frameworks'][i_li]["title"]
                            else:
                                lib_show = lib_show + self.pio_lib['items'][i]['frameworks'][i_li]["title"]
                            i_li = i_li + 1
                        self.lib_list.append(lib_show + "\n")
                    except:
                        stdinit.std_signal_gobal.stdprintln()
                        break
                    i = i + 1
                self.model_1.setStringList(self.lib_list)
                self.listview_1.setModel(self.model_1)

            except:
                stdinit.std_signal_gobal.stdprintln()
        except:
            stdinit.std_signal_gobal.stdprintln()
    def search_lib(self):#-k
        try:
            if self.check_net()==False:
                return False
            if self.search_more_p == 1:
                stdinit.std_signal_gobal.std_process(1, reso.search)
                self.search_load()
                stdinit.std_signal_gobal.std_process(0, reso.search)
                stdinit.std_signal_gobal.std_echo_msg(1, reso.search_sucess)
                self.search_lib_button.setText(reso.lib_find_more)

            else:
                stdinit.std_signal_gobal.std_process(1, reso.search)
                self.search_load()
                stdinit.std_signal_gobal.std_process(0, reso.search)
                stdinit.std_signal_gobal.std_echo_msg(1, reso.search_sucess)
            self.search_lib_button.setEnabled(True)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def find_lib(self):#最早实现方式看码云上的备份
        try:
            if self.check_net()==False:
                return False
            self.search_lib_button.setEnabled(False)
            t1 = threading.Thread(target=self.search_lib, name='th1', args=())
            t1.setDaemon(True)
            t1.start()
        except:
            stdinit.std_signal_gobal.stdprintln()



# class Model(object):
#     def logic(self):
#         data = "Got it"
#         print("Model: Crunching data as per business logic")
#         return data
#
#
# class View(object):
#     def updata(self, data):
#         print("View:Updating the view with results: ", data)
#
#
# class Controller(object):
#     def __init__(self):
#         self.model = Model()
#         self.view = View()
#
#     def interface(self):
#         print("Controller: Relayed the Client asks")
#         data = self.model.logic()
#         self.view.updata(data)




# from cores.download import StdDownload

import click
from cores import stdvarinit
from cores import stdboards
from cores import stdrun
from cores import stdproject
from cores import stdplatform

# class Stduino(App):
#     class Meta:
#         label = 'stduino'
#         handlers = [
#             Base,
#             Project,
#             StdDownload,
#         ]

@stdvarinit.cli.group("Test", short_help="Test manager")
def cli():
    pass

@cli.command("test", short_help="Create a test")
@click.option('--name', prompt='Your name',help='The person to greet.')
@click.option('--output', '-o', multiple=True)
def test(name,output):
    print(name)
    print(output)
    click.echo('Hello %s!' % name)

    pass

@cli.command("pack", short_help="Create a tarball from a package")
@click.option('--name', prompt='Your name',help='The person to greet.')
@click.option('--output', '-o', multiple=True)
def pack(name,output):
    print(name)
    print(output)
    click.echo('Hello %s!' % name)
    click.echo('\n'.join(output))
    pass

# @cli.command()
# @click.option('--password', prompt=True, hide_input=True,
#               confirmation_prompt=False)#可验证
# def encrypt(password):
#     print(password)
    #click.echo('Encrypting password to %s' % password.encode('rot13'))




# with Stduino() as app:
#     app.run()
if __name__ == '__main__':
    stdvarinit.cli()




# -*- coding: utf-8 -*-
"""
Copyright (c) 2015-2021 Stduino.
Released under the GNU GPL3 license.

For more information check the 'LICENSE.txt' file or search www.stduino.com.
For complete license information of the dependencies, check the 'additional_licenses' directory.
"""

# from six.moves import urllib  # 下载文件

#import zipfile  # 解压文件
# import shutil  # 用于压缩
# import subprocess
import os,stat

# import re
# import socket
#
# import zipfile

import time

from PyQt5.QtWidgets import QMessageBox,QComboBox,QWidget,QPushButton,QVBoxLayout,QGridLayout,QTextEdit,QHBoxLayout,QGroupBox,QCheckBox,QAbstractItemView,QListView

from PyQt5.QtCore import QStringListModel,pyqtSignal, QThread

# from function.conf import res
from .stdmsg import reso
from .stdedit import stdinit
import threading
from ..piobuilder.piopltinstall import PioPlatformInstall
from urllib import request

# fo = open("./appearance/stduino_json/library.json", mode='r', encoding='UTF-8')
# st = fo.read()
# fo.close()
# libraries = json.loads(st)

class stdthread(QThread):
    std_signal = pyqtSignal(str)
    def __init__(self):
        super(stdthread, self).__init__()
    def run(self):
        try:
            if stdinit.callch == 1:
                self.install_pio()

            elif stdinit.callch == 2:

                self.install_p()
        except:
            stdinit.std_signal_gobal.stdprintln()
        #a = (1, 2)
        #self.mysignal.emit(a)  # 发射自定义信号

    def install_pio(self):
        try:
            if stdinit.Piopalt_install.isinstalled_plat(stdinit.plat_choose_name):
                stdinit.std_signal_gobal.std_process(1, reso.package_uninstall)

                if stdinit.Piopalt_install.platform_uninstall(stdinit.plat_choose_name):
                    self.std_signal.emit(reso.install_lib)

                    stdinit.std_signal_gobal.std_process(0, reso.package_uninstall)

                else:
                    self.install.setEnabled(True)
                    self.std_signal.emit(reso.install_lib)
                    stdinit.std_signal_gobal.std_process(0, reso.package_uninstall)



            else:
                stdinit.std_signal_gobal.std_process(1, reso.downloading)
                if stdinit.Piopalt_install.platform_install(stdinit.plat_choose_name):
                    self.std_signal.emit(reso.package_uninstall)
                    stdinit.std_signal_gobal.std_process(0, reso.downloading)

                else:

                    self.std_signal.emit(reso.package_uninstall)
                    stdinit.std_signal_gobal.std_process(0, reso.downloading)

            pass
        except:
            stdinit.std_signal_gobal.stdprintln()

    def install_p(self):
        try:
            for i in range(0, 5):
                print(2)
                time.sleep(1)
        except:
            stdinit.std_signal_gobal.stdprintln()

class StdAddPlatforms(QWidget):  # 安装库文件



    def __init__(self):
        try:
            super(StdAddPlatforms, self).__init__()

            self.plat = PioPlatformInstall()
            self.add_lib_s_load_f = 0
            self.find_more_p = 1
            self.pio_libs = []
            self.pio_libs.append(0)
            self.lib_list = []

            self.initUi()
        except:
            stdinit.std_signal_gobal.stdprintln()

    def check_net(self):
        try:
            request.urlopen(url="https://www.baidu.com", timeout=3.0)
            # print(ret)
        except:
            stdinit.std_signal_gobal.std_echo_msg(1,reso.no_net)
            return False
        return True
    def initUi(self):
        try:
            self.bigEditor = QTextEdit()
            self.bigEditor.setReadOnly(True)

            #self.createGridGroupBox()
            self.createGridGroupBox2()
            self.creatVboxGroupBox()
            self.creatVboxGroupBox2()
            mainLayout = QVBoxLayout()
            hboxLayout = QHBoxLayout()
            hboxLayout.addWidget(self.vboxGroupBox2)
            hboxLayout.addWidget(self.vboxGroupBox)

            #the_window.add_lib_s_load.connect(self.libr_load)

            self.install = QPushButton(reso.install_lib, self)

            # self.find_more = QPushButton(reso.lib_find_more, self)
            # self.find_more.setEnabled(False)

            self.install_return = QPushButton(reso.library_installer, self)

            # self.find_more.setDisabled(True)


            self.install.setDisabled(True)

            #self.find_more.clicked.connect(self.find_mores)

            self.install.clicked.connect(self.install_url)
            self.install_return.clicked.connect(self.install_ret)
            #self.install.setFixedWidth(120)
            #self.pro1 = QProgressBar(self)
            # self.pro1.setOrientation(Qt.Horizontal)
            # self.pro1.setMinimum(0)
            # self.pro1.setMaximum(0)
            #self.pro1.setVisible(False)

            glayout = QGridLayout()
            glayout.addWidget(self.install, 0, 0)
            #glayout.addWidget(self.find_more, 0, 1)
            glayout.addWidget(self.install_return, 0, 2)


            # 准备四个控件
            gwg = QWidget()
            # 使用四个控件设置局部布局
            gwg.setLayout(glayout)
            # 将四个控件添加到全局布局中


            #mainLayout.addWidget(self.gridGroupBox)
            mainLayout.addLayout(hboxLayout)
            mainLayout.addWidget(self.gridGroupBox2)
            #mainLayout.addWidget(self.pro1)
            mainLayout.addWidget(gwg)
            #mainLayout.addWidget(self.install_return)
            self.setLayout(mainLayout)
        except:
            stdinit.std_signal_gobal.stdprintln()
    def libr_load(self):
        try:
            if self.check_net()==False:
                return False
            if self.add_lib_s_load_f == 0:
                self.add_lib_s_load_f = 1
                t1 = threading.Thread(target=self.platform_load, name='li_load1')
                t1.setDaemon(True)
                t1.start()

        except:
            stdinit.std_signal_gobal.stdprintln()




    def createGridGroupBox2(self):
        try:
            self.gridGroupBox2 = QGroupBox(reso.lib_version)
            self.layout = QGridLayout()
            # nameLabel = QLabel("发布版本时间")
            self.vsion = QCheckBox(reso.select_version, self)
            self.vsion.setChecked(True)
            self.vsion.setEnabled(False)
            # self.vsion.stateChanged.connect(self.check_vsion)
            self.combobox_1 = QComboBox(self)
            self.combobox_1.setDisabled(True)
            self.layout.setSpacing(10)
            self.layout.addWidget(self.vsion, 1, 0)
            self.combobox_1.addItem("")
            self.layout.addWidget(self.combobox_1, 1, 1)
            self.layout.setColumnStretch(1, 10)
            self.gridGroupBox2.setLayout(self.layout)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def creatVboxGroupBox(self):
        try:
            self.vboxGroupBox = QGroupBox(reso.lib_detail)
            layout = QVBoxLayout()
            self.bigEditor.setPlainText(reso.lib_select)
            layout.addWidget(self.bigEditor)
            self.vboxGroupBox.setLayout(layout)
        except:
            stdinit.std_signal_gobal.stdprintln()


    def creatVboxGroupBox2(self):

        try:
            self.vboxGroupBox2 = QGroupBox(reso.platform_list)
            layout = QVBoxLayout()

            self.model_1 = QStringListModel(self)
            self.listview_1 = QListView(self)  # python3
            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)

            self.listview_1.setEditTriggers(QAbstractItemView.NoEditTriggers)
            #self.listview_1.setFixedWidth(600)
            #self.listview_1.setFixedHeight(400)
            self.listview_1.clicked.connect(self.clicked)
            layout.addWidget(self.listview_1)
            self.vboxGroupBox2.setLayout(layout)
                    # self.listview_1.doubleClicked.connect(lambda: self.change_func(self.listview_1))
                    # self.listview_1.doubleClicked.connect(self.cs)

                    #self.listview_1.clicked.connect(self.clicked)



        except:
            self.lib_list = [reso.no_net]
            self.model_1 = QStringListModel(self)
            self.listview_1 = QListView(self)  # python3
            self.model_1.setStringList(self.lib_list)
            self.listview_1.setModel(self.model_1)
            self.listview_1.setEditTriggers(QAbstractItemView.NoEditTriggers)
            #self.listview_1.setFixedWidth(600)
            #self.listview_1.setFixedHeight(400)

            # self.item_list = ['item %s' % i for i in range(51)]  # 1
            stdinit.std_signal_gobal.stdprintln()

    def platform_load(self):
        try:
            if self.check_net()==False:
                return False
            # print(libraries)
            self.install_return.setText("Finding Platforms File...")
            # self.pio_lib=self.lib.lib_search(self.find_more_p)
            self.pio_plts=self.plat.platform_search()
            if self.pio_plts == None:

                target = stdinit.stdenv + "/.stduino/packages/pioenv/pioboards.json"  # self.abs_path + "/tool/packages/pioenv/Scripts/pio.exe"
                if os.path.exists(target):
                    stdinit.std_signal_gobal.std_echo_msg(0, "请重启软件再次打开该界面若还存在该问题，请至插件安装界面卸载并重新安装Platformio！")
                else:
                    stdinit.std_signal_gobal.std_echo_msg(0, "当前插件安装存在文件缺失，请至插件安装界面卸载并重新安装Platformio！")
                return 0
            # print(self.pio_plts)
            #self.pio_libs.append(self.pio_lib)

            try:

                ii = len(self.pio_plts)
                i = 0

                while i < ii:
                    try:#libs['items'][0]['name']
                        lib_show=self.pio_plts[i]['title']+"\nby "+self.pio_plts[i]['ownername']+"    |    License:"+str(self.pio_plts[i]['license'])+"\nSupported frameworks:"
                        i_lib = len(self.pio_plts[i]['frameworks'])
                        i_li = 0
                        while i_li < i_lib:
                            if i_li>0:
                                lib_show = lib_show + ","+ self.pio_plts[i]['frameworks'][i_li]
                            else:
                                lib_show = lib_show + self.pio_plts[i]['frameworks'][i_li]

                            i_li=i_li+1
                        self.lib_list.append(lib_show+"\n")



                        #self.lib_list.append(self.pio_libs[self.find_more_p]['items'][i]['name']+"\nby "+self.pio_libs[self.find_more_p]['items'][i]['authornames'][0]+"    |   Downloads:"+str(self.pio_libs[self.find_more_p]['items'][i]['dllifetime'])+"\n")#

                    except:

                        stdinit.std_signal_gobal.stdprintln()
                        break
                    i = i + 1
                self.model_1.setStringList(self.lib_list)

                self.listview_1.setModel(self.model_1)
                self.install_return.setText(reso.library_installer)
            except:

                stdinit.std_signal_gobal.stdprintln()
                pass
        except:
            stdinit.std_signal_gobal.stdprintln()



    def install_ret(self):
        try:

            stdinit.std_signal_gobal.std_main_ui_change(2)
            # the_window.m_stackedLayout.setCurrentIndex(0)
            # the_window.liAct2.setEnabled(True)

        except:
            stdinit.std_signal_gobal.stdprintln()
    def clicked(self, qModelIndex):

        try:
            if self.check_net()==False:
                return False
            n_lin=qModelIndex.row()
            descrabe = self.pio_plts[n_lin]['description']
            # for i in
            self.combobox_1.clear()
            #self.pio_libs[self.find_more_p]['items'][i]['name']
            stdinit.plat_choose_name=self.pio_plts[n_lin]['name']
            self.combobox_1.addItem(self.pio_plts[n_lin]['versions'][0])
            if stdinit.Piopalt_install.isinstalled_plat(stdinit.plat_choose_name):
                self.install.setDisabled(False)
                self.install.setText(reso.package_uninstall)
                pass
            else:
                self.install.setDisabled(False)
                self.install.setText(reso.install_lib)
                pass
            # if self.find_more.isEnabled() == False:
            #     self.find_more.setDisabled(False)

            self.bigEditor.setText(descrabe)
        except:
            stdinit.std_signal_gobal.stdprintln()

    def remove_readonly(self,func, path, _):
        "Clear the readonly bit and reattempt the removal"
        os.chmod(path, stat.S_IWRITE)
        func(path)
    def thread_return(self,strin):
        self.install.setText(strin)
        self.install.setEnabled(True)
        self.listview_1.setEnabled(True)
    def install_url(self):

        # from multiprocessing import Process
        # p = Process(target=self.install_url1, args=(1,))
        # print(1)
        try:

            self.install.setEnabled(False)
            self.listview_1.setEnabled(False)

            if stdinit.Piopalt_install.isinstalled_plat(stdinit.plat_choose_name):
                ok = QMessageBox.information(self,
                                             "提示",
                                             "确认要卸载" + stdinit.package_choose_name + "？",
                                             QMessageBox.Yes, QMessageBox.No)
                if ok == QMessageBox.No:
                    return 0
            self.my_thread = stdthread()  # 步骤2. 主线程连接子线,同时传递一个值给子线程
            self.my_thread.std_signal.connect(self.thread_return)
            stdinit.callch = 1
            self.my_thread.start()  # 步骤3 子线程开始执行run函数

        except:
            stdinit.std_signal_gobal.stdprintln()
        #t1 = threading.Thread(target=pay_ticket, name='th1')



    def schedule(self,blocknum, blocksize, totalsize):
        try:
            # """
            # blocknum:当前已经下载的块
            # blocksize:每次传输的块大小
            # totalsize:网页文件总大小
            # """
            per = 100 * blocknum * blocksize / totalsize

            if per > 100:
                per = 100
                self.install_return.setText("Installing Library...")


        except:
            stdinit.std_signal_gobal.stdprintln()

    def find_mores(self):
        try:
            if self.check_net()==False:
                return False
            self.install_return.setText("Finding More File...")
            self.find_more_p = self.find_more_p + 1
            self.pio_libs.append(self.lib.lib_search(self.find_more_p))
            ii = len(self.pio_libs[self.find_more_p]['items'])
            i = 0
            while i < ii:
                self.lib_list.append(self.pio_libs[self.find_more_p]['items'][i]['name'] + "\ndescription:" +
                                     self.pio_libs[self.find_more_p]['items'][i]['description'] + "\n")  #
                i = i + 1
            self.model_1.setStringList(self.lib_list)

            self.listview_1.setModel(self.model_1)
            self.install_return.setText(reso.library_installer)
        except:
            stdinit.std_signal_gobal.stdprintln()



    def check_vsion(self):
        try:
            if self.vsion.isChecked():
                self.combobox_1.setDisabled(False)
            else:
                self.combobox_1.setDisabled(True)
        except:
            stdinit.std_signal_gobal.stdprintln()


    # def find_lib(self):
    #     try:
    #
    #         lib_len = len(libraries["libraries"])
    #         self.lib_list = []
    #         self.lib_list_num = []
    #         first_n=""
    #         self.st_ar = 0
    #         try:
    #             ii = len(self.lib)
    #             i = 0
    #             while i < ii:
    #                 try:
    #                     if re.search(self.nameLineEdit.text(), self.lib[i]['lib_name'],flags=re.IGNORECASE) == None:  # search(self.nameLineEdit.text(), libraries["libraries"][i]["name"]) == None:
    #                         pass
    #                     else:
    #                         self.lib_list.append(str(self.lib[i]['lib_name']))
    #
    #                         self.lib_list_num.append(str(i))
    #
    #
    #                     # if self.lib[i]['id']:
    #                     # lib_list.append(str(self.lib[i]['lib_name']))
    #
    #
    #                     # print(c[i]['id'])
    #                 except:
    #                     break
    #                 i = i + 1
    #             self.st_ar = i
    #
    #         except:
    #             pass
    #
    #         for i in range(lib_len):
    #             try:
    #                 if re.search(self.nameLineEdit.text(),libraries["libraries"][i]["name"], flags=re.IGNORECASE)== None:#search(self.nameLineEdit.text(), libraries["libraries"][i]["name"]) == None:
    #                     pass
    #                 else:
    #                     if first_n!=libraries["libraries"][i]["name"]:
    #                         self.lib_list.append(str(libraries["libraries"][i]["name"]))
    #                         self.lib_list_num.append(str(i+self.st_ar))
    #                         first_n=libraries["libraries"][i]["name"]
    #                 # if self.lib[i]['id']:
    #                 # print(c[i]['id'])
    #             except:
    #                 print("Unexpected error:", sys.exc_info()[1])  # 错误内容
    #
    #                 break
    #         # ii = len(self.lib)
    #         # i = 0
    #         # lib_list = []
    #         # while i < ii:
    #         #     try:
    #         #         if re.search(self.nameLineEdit.text(), self.lib[i]['lib_name']) == None:
    #         #             pass
    #         #         else:
    #         #             lib_list.append(str(self.lib[i]['lib_name']))
    #         #     except:
    #         #         break
    #         #     i = i + 1
    #         self.model_1.setStringList(self.lib_list)
    #         self.listview_1.setModel(self.model_1)
    #     except:
    #         QMessageBox.warning(self, "BUG Warning", "Waring|Error:stdlibs00035！\n You can search it in stduino.com",QMessageBox.Yes)
    #


# class Model(object):
#     def logic(self):
#         data = "Got it"
#         print("Model: Crunching data as per business logic")
#         return data
#
#
# class View(object):
#     def updata(self, data):
#         print("View:Updating the view with results: ", data)
#
#
# class Controller(object):
#     def __init__(self):
#         self.model = Model()
#         self.view = View()
#
#     def interface(self):
#         print("Controller: Relayed the Client asks")
#         data = self.model.logic()
#         self.view.updata(data)








from cores import stdvarinit
import os
import click
@stdvarinit.cli.group("project", short_help="project manager")
def cli():
    pass


# @cli.command("init", short_help="Create a project")
# @click.option('--project-dir','-d', help='target_path.')
# @click.option('--board', '-b', help='target_path.')
# @click.option("-O", "--project-option", multiple=True)
# def project_init(project_dir,board,project_option):
#     section = "env:%s" % board
#     print(section)
#     print(project_dir)
#     click.secho(project_dir, fg="cyan", nl=False)
#     click.echo('Hello %s!' % project_dir)
#     if boards.boards_search(board):
#         pass
#     else:
#
#         _install_platform(board)
#         pass

#cmd = self.pio_env + " project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework

@cli.command("init", short_help="Create a project")
@click.option('--project-dir','-d', help='target_path.')
@click.option('--board', '-b', help='target_path.')
@click.option("-O", "--project-option", multiple=True)
def stdproject_init(project_dir,board,project_option):
    stduino_framework("arduino",project_dir,board)


    click.echo(project_option)

    pass

def other_framework():
    pass
def update_project_env(project_dir, environment, project_option):
    if not project_option:
        return
    config = ProjectConfig(
        os.path.join(project_dir, "platformio.ini"), parse_extra=False
    )

    section = "env:%s" % environment
    if not config.has_section(section):
        config.add_section(section)

    for item in project_option:
        if "=" not in item:
            continue
        _name, _value = item.split("=", 1)
        config.set(section, _name.strip(), _value.strip())

    config.save()
def init_include_readme(include_dir):
    if not os.path.isdir(include_dir):
        os.makedirs(include_dir)
    with open(os.path.join(include_dir, "README"), mode="w", encoding="utf8") as fp:
        fp.write(
            """
This directory is intended for project header files.

A header file is a file containing C declarations and macro definitions
to be shared between several project source files. You request the use of a
header file in your project source file (C, C++, etc) located in `src` folder
by including it, with the C preprocessing directive `#include'.

```src/main.c

#include "header.h"

int main (void)
{
 ...
}
```

Including a header file produces the same results as copying the header file
into each source file that needs it. Such copying would be time-consuming
and error-prone. With a header file, the related declarations appear
in only one place. If they need to be changed, they can be changed in one
place, and programs that include the header file will automatically use the
new version when next recompiled. The header file eliminates the labor of
finding and changing all the copies as well as the risk that a failure to
find one copy will result in inconsistencies within a program.

In C, the usual convention is to give header files names that end with `.h'.
It is most portable to use only letters, digits, dashes, and underscores in
header file names, and at most one dot.

Read more about using header files in official GCC documentation:

* Include Syntax
* Include Operation
* Once-Only Headers
* Computed Includes

https://gcc.gnu.org/onlinedocs/cpp/Header-Files.html
""",
        )


def init_lib_readme(lib_dir):
    if not os.path.isdir(lib_dir):
        os.makedirs(lib_dir)
    with open(os.path.join(lib_dir, "README"), mode="w", encoding="utf8") as fp:
        fp.write(
            """
待完善 更多信息请至wiki.stduino.com浏览
This directory is intended for project specific (private) libraries.
PlatformIO will compile them to static libraries and link into executable file.

The source code of each library should be placed in a an own separate directory
("lib/your_library_name/[here are source files]").

For example, see a structure of the following two libraries `Foo` and `Bar`:

|--lib
|  |
|  |--Bar
|  |  |--docs
|  |  |--examples
|  |  |--src
|  |     |- Bar.c
|  |     |- Bar.h
|  |  |- library.json (optional, custom build options, etc) https://docs.platformio.org/page/librarymanager/config.html
|  |
|  |--Foo
|  |  |- Foo.c
|  |  |- Foo.h
|  |
|  |- README --> THIS FILE
|
|- platformio.ini
|--src
   |- main.c

and a contents of `src/main.c`:
```
#include <Foo.h>
#include <Bar.h>

int main (void)
{
  ...
}

```

Stduino Library Dependency Finder will find automatically dependent
libraries scanning project source files.

More information about Stduino Library Dependency Finder
- https://docs.platformio.org/page/librarymanager/ldf.html
""",
        )


def init_test_readme(test_dir):
    if not os.path.isdir(test_dir):
        os.makedirs(test_dir)
    with open(os.path.join(test_dir, "README"), mode="w", encoding="utf8") as fp:
        fp.write(
            """
This directory is intended for PlatformIO Unit Testing and project tests.

Unit Testing is a software testing method by which individual units of
source code, sets of one or more MCU program modules together with associated
control data, usage procedures, and operating procedures, are tested to
determine whether they are fit for use. Unit testing finds problems early
in the development cycle.

More information about PlatformIO Unit Testing:
- wiki.stduino.com
""",
        )

def init_std_ini(project_dir):
    pio_conetent = """
; Stduino Project Configuration File
;
;   Build options: build flags, source filter
;   Upload options: custom upload port, speed and extra flags
;   Library options: dependencies, extra library storages
;   Advanced options: extra scripting
;
; Please visit documentation for the other options and examples
; wiki.stduino.com

[env:w806duino]
platform = w806
platforms = stduino
board = w806
framework = arduino
debug_tool = cklink
upload_protocol = serial"""
    conf_path = os.path.join(project_dir, "stduino.ini")
    if os.path.isfile(conf_path):
        return
    with open(conf_path, mode="w", encoding="utf8") as fp:
        fp.write(pio_conetent)
    pass
def init_cvs_ignore(project_dir):

    conf_path = os.path.join(project_dir, ".gitignore")
    if os.path.isfile(conf_path):
        return
    with open(conf_path, mode="w", encoding="utf8") as fp:
        fp.write(".pio\n.std\n")
def init_main_other(main_dir):
    if not os.path.isdir(main_dir):
        os.makedirs(main_dir)
    conf_path = os.path.join(main_dir, "main.cpp")
    if os.path.isfile(conf_path):
        return
    main_content ="""

#include "header.h"

int main (void)
{

}
    """
    with open(conf_path, mode="w", encoding="utf8") as fp:
        fp.write(main_content)
def init_main_arduino(main_dir):
    if not os.path.isdir(main_dir):
        os.makedirs(main_dir)
    conf_path = os.path.join(main_dir, "main.cpp")
    if os.path.isfile(conf_path):
        return
    main_content = "\n".join(
        [
            "#include <Arduino.h>",
            "",
            "void setup() {",
            "  // put your setup code here, to run once:",
            "}",
            "",
            "void loop() {",
            "  // put your main code here, to run repeatedly:",
            "}",
            "",
        ]
    )
    with open(conf_path, mode="w", encoding="utf8") as fp:
        fp.write(main_content.strip())

def stduino_framework(framework,projects_dir,board):
    try:
        target_path = projects_dir
        # 大小写区分待解决 win下
        if not os.path.isdir(target_path):
            os.makedirs(target_path)
        # 大小写区分待解决 win下
        # if os.path.exists(target_path):
        #     pass
        # else:
        #     os.makedirs(target_path)
        #target_path = '"' + target_path + '"'
        #main_content = None
        if framework == "arduino":
            init_main_arduino(os.path.join(projects_dir, "src"))
            init_lib_readme(os.path.join(projects_dir, "lib"))
            init_test_readme(os.path.join(projects_dir, "test"))
            init_include_readme(os.path.join(projects_dir, "include"))
            init_cvs_ignore(projects_dir)
            init_std_ini(projects_dir)
            return True
        else:
            init_main_other(os.path.join(projects_dir, "src"))
            init_lib_readme(os.path.join(projects_dir, "lib"))
            init_test_readme(os.path.join(projects_dir, "test"))
            init_include_readme(os.path.join(projects_dir, "include"))
            init_cvs_ignore(projects_dir)
            init_std_ini(projects_dir)
            return True

        # if upload_m == "Disable":  # upload_m 调试前根据这个进行判断是否可以支持调试
        #     cmd = self.pio_env + " project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework
        # else:
        #     cmd = self.pio_env + " project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework + " -O debug_tool=" + upload_m
        #
        # # if upload_m=="Disable":#upload_m 调试前根据这个进行判断是否可以支持调试
        # #     cmd = stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework
        # # else:
        # #     cmd = stdinit.stdenv + "/.stduino/packages/pioenv/Scripts/pio project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework + " -O debug_tool=" + upload_m
        # #
        #
        # proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
        # for line in iter(proc.stdout.readline, b''):
        #     try:
        #         s1 = str(line, encoding='gbk')
        #     except:
        #         s1 = str(line)
        #     # print(s1)
        #     if not subprocess.Popen.poll(proc) is None:
        #         if line == "":
        #             break
        # proc.stdout.close()

        #return self.generate_project_main(framework)
    except:
        #stdinit.std_signal_gobal.stdprintln()
        return False

def _install_platform(board):
    pass

#self.pio_env + " project init -d " + target_path + " --board " + stdinit.board_id + " -O framework=" + framework

    pass
if __name__ == '__main__':
    stdvarinit.cli()

